@@ -52,39 +52,43 @@
 /* #include <unistd.h> */
 #include <time.h>
 
-/* une grande partie des modules utiliss ici consiste en
-   une modification (L. Billard 1997 - 2001) de modules tirs du 
-   'package' ImageMagick de cristy@dupont.com.
-   
-   Le but est de rduire le nombre de couleurs  256 au plus
-   pour pouvoir sortir une image GIF
-  
-*******************************************************************************/
-
-/*
-
-  Copyright 1994 E. I. du Pont de Nemours & Company
-
-  Permission to use, copy, modify, distribute, and sell this software and
-  its documentation for any purpose is hereby granted without fee,
-  provided that the above copyright notice appear in all copies and that
-  both that copyright notice and this permission notice appear in
-  supporting documentation, and that the name of E. I. du Pont de Nemours
-  & Company not be used in advertising or publicity pertaining to
-  distribution of the software without specific, written prior
-  permission.  E. I. du Pont de Nemours & Company makes no representations
-  about the suitability of this software for any purpose.  It is provided
-  "as is" without express or implied warranty.
-
-  E. I. du Pont de Nemours & Company disclaims all warranties with regard
-  to this software, including all implied warranties of merchantability
-  and fitness, in no event shall E. I. du Pont de Nemours & Company be
-  liable for any special, indirect or consequential damages or any
-  damages whatsoever resulting from loss of use, data or profits, whether
-  in an action of contract, negligence or other tortious action, arising
-  out of or in connection with the use or performance of this software.
-
-*******************************************************************************/
+/**
+ * SECTION:dumpToGif
+ * @short_description: add an export capability into GIF files.
+ *
+ * <para>This provides a write routine to export V_Sim views into GIF
+ * files.</para>
+ *
+ * <para>Most of the routines used there have been modified by
+ * L. Billard (1997 - 2001) from the original ones taken from the
+ * ImageMagick package of cristy@dupont.com.</para>
+ *
+ * <para>The goal of this is to reduce the colour span into 256 to be
+ * able to output as GIF file.</para>
+ *
+ * <note>
+ * <para>Copyright 1994 E. I. du Pont de Nemours & Company</para>
+ * <para>Permission to use, copy, modify, distribute, and sell this
+ * software and its documentation for any purpose is hereby granted
+ * without fee, provided that the above copyright notice appear in all
+ * copies and that both that copyright notice and this permission
+ * notice appear in supporting documentation, and that the name of
+ * E. I. du Pont de Nemours & Company not be used in advertising or
+ * publicity pertaining to distribution of the software without
+ * specific, written prior permission.  E. I. du Pont de Nemours &
+ * Company makes no representations about the suitability of this
+ * software for any purpose.  It is provided "as is" without express
+ * or implied warranty.</para>
+ * <para>E. I. du Pont de Nemours & Company disclaims all warranties
+ * with regard to this software, including all implied warranties of
+ * merchantability and fitness, in no event shall E. I. du Pont de
+ * Nemours & Company be liable for any special, indirect or
+ * consequential damages or any damages whatsoever resulting from loss
+ * of use, data or profits, whether in an action of contract,
+ * negligence or other tortious action, arising out of or in
+ * connection with the use or performance of this software.</para>
+ * </note>
+ */
 
 static unsigned char *image;
 
@@ -140,12 +144,12 @@ static Cube cube;
 
 static unsigned int tree_depth = 8;
 
-static unsigned int Assignment(GString *buffer);
-static unsigned int Classification(GString *buffer);
+static unsigned int Assignment();
+static unsigned int Classification(GError **error);
 static void ClosestColor(register Node *node);
 static void Map(register Node *node);
-static unsigned int DitherImage(GString *buffer);
-static unsigned int InitializeCube(unsigned int number_pixels, GString *buffer);
+static unsigned int DitherImage();
+static unsigned int InitializeCube(unsigned int number_pixels, GError **error);
 static Node *InitializeNode(
   unsigned int id,unsigned int level,Node *parent,
   unsigned int mid_red,unsigned int mid_green,unsigned int mid_blue);
@@ -157,10 +161,10 @@ static void Reduction(unsigned int number_colors);
 static unsigned int LZWEncodeImage(unsigned int data_size);
 static void LSBFirstWriteShort(unsigned int value);
 
-int writeViewInGifFormat(FileFormat *format, GString *buffer,
-			 char* filename, int width, int height,
-			 VisuData *dataObj, guchar* imageData,
-			 voidDataFunc functionWait, gpointer data);
+static gboolean writeViewInGifFormat(FileFormat *format, const char* filename,
+				     int width, int height, VisuData *dataObj,
+				     guchar* imageData, GError **error,
+				     voidDataFunc functionWait, gpointer data);
 
 
 /******************************************************************************/
@@ -173,27 +177,21 @@ void dumpToGif_setImage(Image *data)
 
 /******************************************************************************/
 
-unsigned int Assignment(GString *buffer) {
+unsigned int Assignment() {
 
 
-  img->colormap=(ColorPacket *)
-    malloc((unsigned int) cube.colors*sizeof(ColorPacket));
-  if (img->colormap == (ColorPacket *) NULL) {
-    g_string_append(buffer, _("Unable to quantize image\n"
-			      "Memory allocation failed for img->colormap\n"));
-      return 1;
-  }
+  img->colormap=g_malloc(cube.colors*sizeof(ColorPacket));
   cube.colormap=img->colormap;
   cube.colors=0;
   Map(cube.root);
   img->colors=(unsigned int) cube.colors;
-  if (DitherImage(buffer)) return 1;
+  if (DitherImage()) return 1;
   return 0;
 }
 
 /******************************************************************************/
 
-unsigned int Classification(GString *buffer) {
+unsigned int Classification(GError **error) {
 
   register unsigned int i;
   register Node *node;
@@ -221,13 +219,13 @@ unsigned int Classification(GString *buffer) {
             node->mid_red+(id & 1 ? bisect : -bisect),
             node->mid_green+(id & 2 ? bisect : -bisect),
             node->mid_blue+(id & 4 ? bisect : -bisect));
-          if (node->child[id] == (Node *) NULL) {
-	    g_string_append_printf(buffer, _("Unable to quantize image\n"
-					     "Memory allocation failed"
-					     " for node->child[%d]\n"),
-                             id);
+          if (node->child[id] == (Node *) NULL)
+	    {
+	      *error = g_error_new(VISU_ERROR_DUMP, DUMP_ERROR_ENCODE,
+				   _("Unable to quantize image, "
+				     "initialisation failed for node child %d."), id);
               return 1;
-          }
+	    }
           if (level == cube.depth) cube.colors++;
       }
       /*
@@ -305,7 +303,7 @@ void Map(register Node *node) {
  
 /******************************************************************************/
 
-unsigned int DitherImage(GString *buffer) {
+unsigned int DitherImage() {
 
 #define MaxError  16
 
@@ -325,15 +323,9 @@ unsigned int DitherImage(GString *buffer) {
   unsigned short index;
 
   
-  cache=(int *) malloc((1 << 18)*sizeof(int));
-  error=(ErrorPacket *) malloc(((img->columns+2) << 1)*sizeof(ErrorPacket));
-  range_table=(unsigned char *) malloc(3*(MaxRGB+1)*sizeof(unsigned char));
-  if ((cache == (int *) NULL) || (error == (ErrorPacket *) NULL) ||
-      (range_table == (unsigned char *) NULL)) {
-    g_string_append(buffer, _("Unable to dither image\n"
-			      "Memory allocation failed in DitherImage\n"));
-      return 1;
-  }
+  cache=g_malloc((1 << 18)*sizeof(int));
+  error=g_malloc(((img->columns+2) << 1)*sizeof(ErrorPacket));
+  range_table=g_malloc(3*(MaxRGB+1)*sizeof(unsigned char));
   
   for (i=0; i < (1 << 18); i++) cache[i]=(-1);
   for (i=0; i < ((img->columns+2) << 1); i++) {
@@ -424,15 +416,15 @@ unsigned int DitherImage(GString *buffer) {
     }
   }
   
-  (void)free(range_table);
-  (void)free(error);
-  (void)free(cache);
+  g_free(range_table);
+  g_free(error);
+  g_free(cache);
   return 0;
 }
 
 /******************************************************************************/
 
-unsigned int InitializeCube(unsigned int number_pixels, GString *buffer) {
+unsigned int InitializeCube(unsigned int number_pixels, GError **error) {
   
   char c;
   register int i;
@@ -454,11 +446,12 @@ unsigned int InitializeCube(unsigned int number_pixels, GString *buffer) {
   /* Initialize root node. */
   cube.root=InitializeNode(0,0,(Node *) NULL,
      (MaxRGB+1) >> 1,(MaxRGB+1) >> 1, (MaxRGB+1) >> 1);
-  if (cube.root == (Node *) NULL) {
-    g_string_append(buffer, _("Unable to quantize image\n"
-			      "Memory allocation failed for cube.root\n"));
+  if (cube.root == (Node *) NULL)
+    {
+      *error = g_error_new(VISU_ERROR_DUMP, DUMP_ERROR_ENCODE,
+			   _("Unable to quantize image, initialisation failed."));
       return 1;
-  }
+    }
   cube.root->parent=cube.root;
   cube.root->number_colors=(unsigned long) (~0);
   cube.colors=0;
@@ -587,7 +580,7 @@ void Reduction(unsigned int number_colors)
 
 /******************************************************************************/
 
-unsigned int dumpToGif_quantizeImage(unsigned int number_colors, GString *buffer,
+unsigned int dumpToGif_quantizeImage(unsigned int number_colors, GError **error,
 				     voidDataFunc functionWait, gpointer data)
 {
   Nodes *nodes;
@@ -597,11 +590,11 @@ unsigned int dumpToGif_quantizeImage(unsigned int number_colors, GString *buffer
 
   /* Reduce the number of colors in the continuous tone image. */
   if (number_colors > MaxMapSize) number_colors=MaxMapSize;
-  if(InitializeCube(img->columns*img->rows, buffer)) return 1;
+  if(InitializeCube(img->columns*img->rows, error)) return 1;
   
-  if(Classification(buffer)) return 1;
+  if(Classification(error)) return 1;
   Reduction(number_colors);
-  if(Assignment(buffer)) return 1;
+  if(Assignment(error)) return 1;
 
   do {
     nodes=cube.node_queue->next;
@@ -656,12 +649,10 @@ unsigned int LZWEncodeImage(unsigned int data_size) {
 
   
 
-  packet=(unsigned char *) malloc(256*sizeof(unsigned char));
-  hash_code=(short *) malloc(MaxHashTable*sizeof(short));
-  hash_prefix=(short *) malloc(MaxHashTable*sizeof(short));
-  hash_suffix=(unsigned char *) malloc(MaxHashTable*sizeof(unsigned char));
-  if ((!packet) || (!hash_code) || (!hash_prefix) || (!hash_suffix))
-     return(False);
+  packet=g_malloc(256*sizeof(unsigned char));
+  hash_code=g_malloc(MaxHashTable*sizeof(short));
+  hash_prefix=g_malloc(MaxHashTable*sizeof(short));
+  hash_suffix=g_malloc(MaxHashTable*sizeof(unsigned char));
   
   number_bits=data_size;
   max_code=MaxCode(number_bits);
@@ -739,10 +730,10 @@ unsigned int LZWEncodeImage(unsigned int data_size) {
       (void) fwrite((char *) packet,1,byte_count,file);
   }
   
-  (void)free(hash_suffix);
-  (void)free(hash_prefix);
-  (void)free(hash_code);
-  (void)free(packet);
+  g_free(hash_suffix);
+  g_free(hash_prefix);
+  g_free(hash_code);
+  g_free(packet);
   
   if (i < img->packets) return(False);
   return(True);
@@ -821,17 +812,15 @@ DumpType* dumpToGif_init()
   #define descrGIF _("Gif (256 colors) file")
   FileFormat* fmt;
 
-  gif = malloc(sizeof(DumpType));
-  if (!gif)
-    {
-      allocationProblems();
-      exit(1);
-    }
+  gif = g_malloc(sizeof(DumpType));
   fmt = fileFormatNew(descrGIF, typeGIF);
   if (!fmt)
-    g_error("Can't initialize the GIF dump module, aborting.\n");
+    {
+      g_error("Can't initialize the GIF dump module, aborting.\n");
+    }
 
   gif->bitmap = TRUE;
+  gif->hasAlpha = FALSE;
   gif->fileType = fmt;
   gif->writeFunc = writeViewInGifFormat;
 
@@ -842,10 +831,10 @@ DumpType* dumpToGif_init()
 }
 
 
-int writeViewInGifFormat(FileFormat *format, GString *buffer,
-			 char* filename, int width, int height,
-			 VisuData *dataObj, guchar* imageData,
-			 voidDataFunc functionWait, gpointer data)
+static gboolean writeViewInGifFormat(FileFormat *format _U_, const char* filename,
+				     int width, int height, VisuData *dataObj _U_,
+				     guchar* imageData, GError **error,
+				     voidDataFunc functionWait, gpointer data)
 {
   register unsigned int i;
   register ColorPacket *q;
@@ -853,35 +842,30 @@ int writeViewInGifFormat(FileFormat *format, GString *buffer,
   unsigned char bits_per_pixel, c;
   unsigned int status;
 
-  image = imageData;
-  if (!image)
-    return 1;
+  g_return_val_if_fail(error && !*error, FALSE);
+  g_return_val_if_fail(imageData, FALSE);
+
+  waitData = data;
+  waitFunc = functionWait;
+  image    = imageData;
 
   DBG_fprintf(stderr, "Dump Gif : begin export in %dx%d...\n", width, height);
 
   file = fopen(filename, "wb");
   if(!file)
     {
-      g_string_append(buffer, _("Cannot open file (to write in)\n"));
-      free(image);
-      return 1;
+      *error = g_error_new(VISU_ERROR_DUMP, DUMP_ERROR_FILE,
+			   _("Cannot open file (to write in)."));
+      return FALSE;
     }
 
-  waitData = data;
-  waitFunc = functionWait;
-
   img=g_malloc(sizeof(Image));
+  img->colormap = (ColorPacket*)0;
 
   img->columns = width;
   img->rows = height;
   img->packets=img->columns*img->rows;
-  img->pixels=(ColorPacket *)
-    malloc((unsigned int) img->packets*sizeof(ColorPacket));
-  if (!img->pixels)
-    {
-      g_string_append(buffer, _("Memory allocation error for img->packets\n"));
-      return 1;
-    }
+  img->pixels=g_malloc(img->packets*sizeof(ColorPacket));
   q=img->pixels;
   p=image;
   for (i=0; i < img->packets; i++) {
@@ -892,8 +876,14 @@ int writeViewInGifFormat(FileFormat *format, GString *buffer,
     q++;
   }
   
-  if(dumpToGif_quantizeImage(256, buffer, waitFunc, data))
-    return 1;
+  if(dumpToGif_quantizeImage(256, error, waitFunc, data))
+    {
+      g_free(img->pixels);
+      if (img->colormap)
+	g_free(img->colormap);
+      g_free(img);
+      return FALSE;
+    }
   dumpToGif_syncImage();
    
   for (bits_per_pixel=1; bits_per_pixel < 8; bits_per_pixel++)
@@ -922,8 +912,7 @@ int writeViewInGifFormat(FileFormat *format, GString *buffer,
     (void) fputc(0x0,file);
   }
   /* criture du commentaire */
-  write_comment_comm( 
-		     (unsigned char *)"Image cre par le programme V_Sim\n"
+  write_comment_comm((unsigned char *)"Image cre par le programme V_Sim\n"
 		     "Auteur : L. BILLARD");
       
   (void) fputc(',',file);  /* image separator */
@@ -937,19 +926,23 @@ int writeViewInGifFormat(FileFormat *format, GString *buffer,
   c=Max(bits_per_pixel,2);
   (void) fputc((int)(char) c,file);
   status=LZWEncodeImage(Max(bits_per_pixel,2)+1);
-  if (status == False) {
-    g_string_append(buffer, _("Unable to write image\n"
-			      "Memory allocation failed for LZWEncodeImage\n"));
-    return 1;
+  if (status == False)
+    {
+      *error = g_error_new(VISU_ERROR_DUMP, DUMP_ERROR_ENCODE,
+			   _("Fail to compress the GIF file."));
+      g_free(img->pixels);
+      g_free(img->colormap);
+      g_free(img);
+      return FALSE;
   }
   (void) fputc(0x0,file);
   (void) fputc(';',file); /* terminator */
    
   (void) fclose(file);
    
-  (void)free(img->pixels);
-  (void)free(img->colormap);
-  (void)free(img);
+  g_free(img->pixels);
+  g_free(img->colormap);
+  g_free(img);
   
-  return 0;
+  return TRUE;
 }
