@@ -106,8 +106,7 @@ static gboolean readAxesLineStipple(gchar **lines, int nbLines,
 
 /* Export function that is called by visu_module to write the
    values of resources to a file. */
-static gboolean exportResourcesAxes(GString *data, int *nbLinesWritten,
-				    VisuData *dataObj);
+static void exportResourcesAxes(GString *data, VisuData *dataObj);
 
 
 static OpenGLExtension* extensionAxes;
@@ -123,7 +122,6 @@ static void onAxesParametersChange(VisuData *dataObj, OpenGLView *view, gpointer
 static void onAxesConfigChange(GObject *visu, VisuData *dataObj, gpointer data);
 static void rebuildAxesOnResourcesLoaded(GObject *visu, VisuData *dataObj,
 					 gpointer data);
-static void onDataReadySignal(GObject *visu, VisuData *dataObj, gpointer data);
 static void onDataNewSignal(GObject *visu, VisuData *dataObj, gpointer data);
 
 OpenGLExtension* initExtensionAxes()
@@ -166,8 +164,6 @@ OpenGLExtension* initExtensionAxes()
   axesLineWidth   = RESOURCE_AXES_LINE_DEFAULT;
   axesLineStipple = RESOURCE_AXES_STIPPLE_DEFAULT;
 
-  g_signal_connect(VISU_INSTANCE, "dataReadyForRendering",
-		   G_CALLBACK(onDataReadySignal), (gpointer)0);
   g_signal_connect(VISU_INSTANCE, "dataNew",
 		   G_CALLBACK(onDataNewSignal), (gpointer)0);
 
@@ -278,15 +274,6 @@ static void rebuildAxesOnResourcesLoaded(GObject *visu _U_, VisuData *dataObj,
     return;
   axesDraw(dataObj);
 }
-static void onDataReadySignal(GObject *visu _U_, VisuData *dataObj,
-			      gpointer data _U_)
-{
-  if (!dataObj)
-    return;
-
-  axesHaveBeenBuilt = FALSE;
-  axesDraw(dataObj);
-}
 static void onDataNewSignal(GObject *visu _U_, VisuData *dataObj,
 			    gpointer data _U_)
 {
@@ -295,6 +282,19 @@ static void onDataNewSignal(GObject *visu _U_, VisuData *dataObj,
   g_signal_connect(G_OBJECT(dataObj), "OpenGLWidthHeight",
 		   G_CALLBACK(onAxesParametersChange), (gpointer)0);
 }
+static void onAxesConfigChange(GObject *visu _U_, VisuData *dataObj, gpointer data)
+{
+  g_return_if_fail(dataObj);
+
+  onAxesParametersChange(dataObj, visuDataGet_openGLView(dataObj), data);
+}
+
+static void onAxesParametersChange(VisuData *dataObj, OpenGLView *view _U_,
+				   gpointer data _U_)
+{
+  axesHaveBeenBuilt = FALSE;
+  axesDraw(dataObj);
+}
 
 /**
  * drawAxes
@@ -316,9 +316,9 @@ void drawAxes(float length, GLsizei w, GLsizei h, float width,
   glVertex3f(0.0f, 0.0f, long_axes ? -length : 0); glVertex3f(0.0f, 0.0f, length);
   glEnd();
 
-  glRasterPos3f(length, 0.0f, 0.0f); openGLText_drawChars("x"); 
-  glRasterPos3f(0.0f, length, 0.0f); openGLText_drawChars("y"); 
-  glRasterPos3f(0.0f, 0.0f, length); openGLText_drawChars("z");
+  glRasterPos3f(length, 0.0f, 0.0f); openGLText_drawChars("x", TEXT_NORMAL); 
+  glRasterPos3f(0.0f, length, 0.0f); openGLText_drawChars("y", TEXT_NORMAL); 
+  glRasterPos3f(0.0f, 0.0f, length); openGLText_drawChars("z", TEXT_NORMAL);
 
   if(legend != NULL)
     {
@@ -329,7 +329,7 @@ void drawAxes(float length, GLsizei w, GLsizei h, float width,
       glMatrixMode (GL_MODELVIEW);
       glPushMatrix();
       glLoadIdentity ();  
-      glRasterPos3f(20., 5., 0.9); openGLText_drawChars(legend);
+      glRasterPos3f(20., 5., 0.9); openGLText_drawChars(legend, TEXT_NORMAL);
       glPopMatrix();
       glMatrixMode(GL_PROJECTION);
       glPopMatrix();
@@ -467,19 +467,6 @@ void draw_coloured_cone(double r, double h, int n, float phi_prime_zero)
 /* /\*   glDisable(GL_POLYGON_SMOOTH); *\/ */
 
 /* } */
-static void onAxesConfigChange(GObject *visu _U_, VisuData *dataObj, gpointer data)
-{
-  g_return_if_fail(dataObj);
-
-  onAxesParametersChange(dataObj, visuDataGet_openGLView(dataObj), data);
-}
-
-static void onAxesParametersChange(VisuData *dataObj, OpenGLView *view _U_,
-				   gpointer data _U_)
-{
-  axesHaveBeenBuilt = FALSE;
-  axesDraw(dataObj);
-}
 
 void axesDraw(VisuData *dataObj)
 {
@@ -497,7 +484,7 @@ void axesDraw(VisuData *dataObj)
   if (!dataObj)
     return;
 
-  DBG_fprintf(stderr, "Extension axes : creating axes.\n");
+  DBG_fprintf(stderr, "Extension axes: creating axes.\n");
 
   view = visuDataGet_openGLView(dataObj);
   length = 0.26178f * view->camera->length0;
@@ -540,7 +527,11 @@ void axesDraw(VisuData *dataObj)
   glLoadIdentity();
   glFrustum(mini, maxi, mini, maxi, near, far);
   glMatrixMode(GL_MODELVIEW);
-  if(getRenderingMethodInUse() == renderingSpinGet_static())
+  glPushMatrix();
+  glTranslated(+view->box->dxxs2+view->camera->centre[0],
+	       +view->box->dyys2+view->camera->centre[1],
+	       +view->box->dzzs2+view->camera->centre[2]);
+  if(visuRenderingClassIs_currentByName(VISU_RENDERING_SPIN))
     {
       float phi_prime_zero, phi_a, theta_a;
 
@@ -551,7 +542,7 @@ void axesDraw(VisuData *dataObj)
       glViewport(xx, yy, w, h);
 
       /* Draw the first color cone */
-      drawAxes(1.5*length, w, h, axesLineWidth, "front", TRUE);
+      drawAxes(1.5*length, w, h, axesLineWidth, _("front"), TRUE);
 
       glPushMatrix();
       glRotatef(phi_a, 0, 0, 1);
@@ -570,14 +561,15 @@ void axesDraw(VisuData *dataObj)
       glCullFace(GL_BACK);
       glPopMatrix();
 
-      drawAxes(1.5*length, w, h, axesLineWidth, "back", TRUE);
+      drawAxes(1.5*length, w, h, axesLineWidth, _("back"), TRUE);
     }
-  else if(getRenderingMethodInUse() == renderingAtomicGet_static())
+  else if(visuRenderingClassIs_currentByName(VISU_RENDERING_ATOMIC))
     {
       glViewport(xx, yy, w, h);
 
       drawAxes(length, w, h, axesLineWidth, NULL, FALSE);
     }
+  glPopMatrix();
   glMatrixMode(GL_PROJECTION);
   glPopMatrix();
   glMatrixMode(GL_MODELVIEW);
@@ -668,8 +660,7 @@ static gboolean readAxesLineStipple(gchar **lines, int nbLines,
 
 /* Export function that is called by visu_module to write the
    values of resources to a file. */
-static gboolean exportResourcesAxes(GString *data, int *nbLinesWritten,
-				    VisuData *dataObj _U_)
+static void exportResourcesAxes(GString *data, VisuData *dataObj _U_)
 {
   g_string_append_printf(data, "# %s\n", DESC_RESOURCE_AXES_USED);
   g_string_append_printf(data, "%s:\n", FLAG_RESOURCE_AXES_USED);
@@ -685,7 +676,4 @@ static gboolean exportResourcesAxes(GString *data, int *nbLinesWritten,
   g_string_append_printf(data, "%s:\n", FLAG_RESOURCE_AXES_STIPPLE);
   g_string_append_printf(data, "    %d\n", axesLineStipple);
   g_string_append_printf(data, "\n");
-  *nbLinesWritten += 10;
-
-  return TRUE;
 }
