@@ -125,7 +125,7 @@ static gboolean readBoxLineStipple(gchar **lines, int nbLines,
 
 /* Export function that is called by visu_module to write the
    values of resources to a file. */
-static gboolean exportResourcesBox(GString *data, int *nbLinesWritten, VisuData *dataObj);
+static void exportResourcesBox(GString *data, VisuData *dataObj);
 
 static void rebuildBox(VisuData *dataObj);
 
@@ -332,8 +332,6 @@ void boxDraw(VisuData *data)
   glColor3f(boxRGB[0], boxRGB[1], boxRGB[2]);
   glDisable(GL_LIGHTING);
   glDisable(GL_DITHER);
-  glPushMatrix();
-  glTranslated(-box->dxxs2, -box->dyys2, -box->dzzs2);
   /* Draw the basic lines. */
   if (boxLineStipple[0] != 65535)
     {
@@ -442,7 +440,6 @@ void boxDraw(VisuData *data)
       if (boxLineStipple[1] != 65535)
 	glDisable(GL_LINE_STIPPLE);
     }
-  glPopMatrix();  
   glEnable(GL_LIGHTING);
   glEnable(GL_DITHER); /* WARNING: it is the default! */
   glLineWidth(1.);
@@ -530,8 +527,7 @@ static gboolean readBoxLineStipple(gchar **lines, int nbLines, int position,
 
 /* Export function that is called by visu_module to write the
    values of resources to a file. */
-static gboolean exportResourcesBox(GString *data, int *nbLinesWritten,
-				   VisuData *dataObj _U_)
+static void exportResourcesBox(GString *data, VisuData *dataObj _U_)
 {
   g_string_append_printf(data, "# %s\n", DESC_RESOURCE_BOX_USED);
   g_string_append_printf(data, "%s:\n", FLAG_RESOURCE_BOX_USED);
@@ -547,7 +543,4 @@ static gboolean exportResourcesBox(GString *data, int *nbLinesWritten,
   g_string_append_printf(data, "%s:\n", FLAG_RESOURCE_BOX_STIPPLE);
   g_string_append_printf(data, "    %d %d\n", boxLineStipple[0], boxLineStipple[1]);
   g_string_append_printf(data, "\n");
-  *nbLinesWritten += 13;
-
-  return TRUE;
 }
