@@ -299,7 +299,6 @@ void isosurfacesOrder_polygons(SurfacesOrder *order, Surfaces *surf[]);
 /**
  * isosurfacesDraw_surfaces:
  * @openGLId: an id for the OpenGL list ;
- * @view: a #OpenGLView object that describes the rendering box.
  * @surf: an array of #Surfaces object, must be NULL terminated.
  * @order: a #SurfacesOrder object.
  *
@@ -308,8 +307,7 @@ void isosurfacesOrder_polygons(SurfacesOrder *order, Surfaces *surf[]);
  * then, the interior of the surfaces is drawn as color inverse.
  *
  */
-void isosurfacesDraw_surfaces(int openGLId, OpenGLView *view,
-			      Surfaces *surf[], SurfacesOrder *order);
+void isosurfacesDraw_surfaces(int openGLId, Surfaces *surf[], SurfacesOrder *order);
 /**
  * isosurfacesDuplicate:
  * @totalList: an OpenGL identifier for the global list to create ;
@@ -331,7 +329,17 @@ void isosurfacesDuplicate(int totalList, int simpleBlockList,
  *
  * Copy the given @box to the box of surfaces.
  */
-void isosurfacesSet_box(Surfaces *surf, float box[6]);
+void isosurfacesSet_box(Surfaces *surf, double box[6]);
+
+/**
+ * isosurfacesGet_box:
+ * @surf: a #Surfaces object.
+ *
+ * Get the box of the surfaces.
+ *
+ * Returns: 6 floats, private, read-only.
+ */
+double* isosurfacesGet_box(Surfaces *surf);
 
 /**
  * isosurfacesGet_floatProperty:
@@ -441,7 +449,7 @@ struct Surfaces_struct
   SurfacesPoints volatilePlanes;
 
   /* The description of the box where surfaces are drawn. */
-  float local_box[6];
+  double local_box[6];
 
   /* Resources for each surfaces. */
   SurfaceResource **resources;
