@@ -41,19 +41,7 @@
 	knowledge of the CeCILL license and that you accept its terms. You can
 	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
 */
-#include <gtk/gtksignal.h>
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtkcelllayout.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkcellrendererpixbuf.h>
-#include <gtk/gtkcolorseldialog.h>
-#include <gtk/gtkexpander.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkhscale.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkimage.h>
+#include <gtk/gtk.h>
 
 #include "gtk_colorComboBoxWidget.h"
 #include <visu_tools.h>
@@ -314,8 +302,8 @@ static void colorComboBox_finalize(GObject *obj)
   DBG_fprintf(stderr, "Gtk ColorComboBox : finalize object %p.\n", (gpointer)obj);
 
   colorComboBox = COLOR_COMBOX(obj);
-  if (colorComboBox->expandRanges)
-    gtk_widget_destroy(colorComboBox->expandRanges);
+/*   if (colorComboBox->expandRanges) */
+/*     gtk_widget_destroy(colorComboBox->expandRanges); */
 
   /* Chain up to the parent class */
   G_OBJECT_CLASS(colorComboBox_parent_class)->finalize(obj);
@@ -347,6 +335,7 @@ static void buildWidgets(ColorComboBox *colorComboBox)
   klass = G_OBJECT_GET_CLASS(colorComboBox);
   gtk_combo_box_set_model(GTK_COMBO_BOX(colorComboBox),
 			  GTK_TREE_MODEL(COLOR_COMBOX_CLASS(klass)->listStoredColors));
+  DBG_fprintf(stderr, "Gtk ColorComboBox: build widgets.\n");
   renderer = gtk_cell_renderer_pixbuf_new();
   gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(colorComboBox), renderer, FALSE);
   if (colorComboBox->hasAlphaChannel)
@@ -365,12 +354,15 @@ static void buildWidgets(ColorComboBox *colorComboBox)
     gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(colorComboBox), renderer,
 				  "text", COLUMN_COLOR_LABEL);
   colorComboBox->rendererRGB = renderer;
+  DBG_fprintf(stderr, " | renderers OK\n");
 
   gtk_combo_box_set_active(GTK_COMBO_BOX(colorComboBox), 1);
+  DBG_fprintf(stderr, " | selection OK\n");
 
   colorComboBox->expandRanges = (GtkWidget*)0;
   if (colorComboBox->withRanges)
     {
+      DBG_fprintf(stderr, "Gtk ColorComboBox: add range widgets.\n");
       rgb[0] = RED_ELE_LABEL;
       rgb[1] = GREEN_ELE_LABEL;
       rgb[2] = BLUE_ELE_LABEL;
@@ -410,6 +402,7 @@ static void buildWidgets(ColorComboBox *colorComboBox)
       gtk_box_pack_start(GTK_BOX(hbox), colorComboBox->addButton, FALSE, FALSE, 2);
       image = gtk_image_new_from_stock ("gtk-add", GTK_ICON_SIZE_BUTTON);
       gtk_container_add(GTK_CONTAINER(colorComboBox->addButton), image);
+      DBG_fprintf(stderr, " | color OK\n");
 
       table = gtk_table_new(3, 4, FALSE);
       gtk_box_pack_start(GTK_BOX(vboxExpand), table, FALSE, FALSE, 5);
@@ -454,6 +447,7 @@ static void buildWidgets(ColorComboBox *colorComboBox)
       gtk_table_attach(GTK_TABLE(table), colorComboBox->rgbRanges[3],
 		       3, 4, i, i + 1,
 		       GTK_EXPAND | GTK_FILL, GTK_SHRINK, 1, 0);
+      DBG_fprintf(stderr, " | material OK\n");
       /* Attach the callbacks. */
       for (i = 0; i < 4; i++)
 	colorComboBox->rgbSignals[i] = 
@@ -465,6 +459,7 @@ static void buildWidgets(ColorComboBox *colorComboBox)
 			   G_CALLBACK(colorComboBox_materialChanged), (gpointer)colorComboBox);
       g_signal_connect(G_OBJECT(colorComboBox->addButton), "clicked",
 		       G_CALLBACK(colorComboBox_addButtonClicked), (gpointer)colorComboBox);
+      DBG_fprintf(stderr, " | signals OK\n");
     }
   colorComboBox->comboSignal = g_signal_connect(G_OBJECT(colorComboBox), "changed",
 						G_CALLBACK(colorComboBox_changed),
@@ -475,7 +470,7 @@ GtkWidget* colorComboBox_newWithRanges(gboolean hasAlphaChannel)
 {
   ColorComboBox *colorComboBox;
 
-  DBG_fprintf(stderr, "Gtk ColorComboBox : creating new object with alpha & ranges : %d.\n",
+  DBG_fprintf(stderr, "Gtk ColorComboBox: creating new object with alpha & ranges : %d.\n",
 	      hasAlphaChannel);
 
   colorComboBox = COLOR_COMBOX(g_object_new(colorComboBox_get_type (), NULL));
@@ -484,6 +479,7 @@ GtkWidget* colorComboBox_newWithRanges(gboolean hasAlphaChannel)
 
   buildWidgets(colorComboBox);
 
+  DBG_fprintf(stderr, "Gtk ColorComboBox: object ready.\n");
   return GTK_WIDGET(colorComboBox);
 }
 
@@ -491,7 +487,7 @@ GtkWidget* colorComboBox_new(gboolean hasAlphaChannel)
 {
   ColorComboBox *colorComboBox;
 
-  DBG_fprintf(stderr, "Gtk ColorComboBox : creating new object with alpha : %d.\n",
+  DBG_fprintf(stderr, "Gtk ColorComboBox: creating new object with alpha : %d.\n",
 	      hasAlphaChannel);
 
   colorComboBox = COLOR_COMBOX(g_object_new(colorComboBox_get_type (), NULL));
@@ -500,6 +496,7 @@ GtkWidget* colorComboBox_new(gboolean hasAlphaChannel)
 
   buildWidgets(colorComboBox);
 
+  DBG_fprintf(stderr, "Gtk ColorComboBox: object ready.\n");
   return GTK_WIDGET(colorComboBox);
 }
 
