@@ -47,7 +47,7 @@
 #include <glib.h>
 #include <glib-object.h>
 
-#include <gtk/gtkentry.h>
+#include <gtk/gtk.h>
 
 #include <visu_tools.h>
 
