@@ -51,11 +51,13 @@
 #include "visu_basic.h"
 #include "visu_plugins.h"
 
-#define V_SIM_LICENCE_FILE  _("licence.en.txt")
+#define V_SIM_LICENCE_FILE   _("licence.en.txt")
 /* Translate if there is a authors file in another language. */
-#define V_SIM_AUTHORS_FILE  _("authors")
+#define V_SIM_AUTHORS_FILE   _("authors")
 /* Translate if there is a readme file in another language. */
-#define V_SIM_README_FILE   _("readme")
+#define V_SIM_README_FILE    _("readme")
+/* Translate if there is a readme file in another language. */
+#define V_SIM_CHANGELOG_FILE _("ChangeLog.en")
 
 enum
   {
@@ -71,17 +73,18 @@ static void buildPluginsTab();
 /* 			     gpointer user_data); */
 /* static gboolean onMouseMotion(GtkWidget *wd, GdkEventMotion *event, */
 /* 			      gpointer user_data); */
+static void setChangelog(const gchar* buffer, gssize size, GtkTextBuffer *text);
 
 
 /* create the window. */
 void gtkAboutBuild_window(GtkMain *main)
 {
-  GtkWidget *wd;
+  GtkWidget *wd, *note;
   gchar *buffer, *utf8Buffer, *start, *end;
   GtkTextBuffer *text;
   GtkTextIter startIter, endIter;
   GtkTextTag *monoTag, *boldTag, *linkTag;
-  gsize taille;
+  gsize taille, size;
   GError *err;
   gchar *chemin;
 
@@ -105,8 +108,8 @@ void gtkAboutBuild_window(GtkMain *main)
 		       V_SIM_WEB_SITE"</u></span>");
 /* #endif */
 
-  wd = lookup_widget(main->aboutDialog, "notebookAbout");
-  gtk_widget_set_name(wd, "message_notebook");
+  note = lookup_widget(main->aboutDialog, "notebookAbout");
+  gtk_widget_set_name(note, "message_notebook");
 
   /* Create the interior of the tabs. */
   chemin = g_build_filename(V_SIM_LEGAL_DIR, V_SIM_LICENCE_FILE, NULL);
@@ -168,6 +171,28 @@ void gtkAboutBuild_window(GtkMain *main)
     }
   g_free(chemin);
 
+  chemin = g_build_filename(V_SIM_LEGAL_DIR, V_SIM_CHANGELOG_FILE, NULL);
+  buffer = (gchar*)0;
+  err = (GError*)0;
+  if (!g_file_get_contents(chemin, &buffer, &taille, &err))
+    g_warning("Can't find the changelog file, normally"
+	      " it should be in '%s'.\n", chemin);
+  else
+    {
+      utf8Buffer = g_convert(buffer, taille, "UTF-8", "ISO-8859-1", NULL, &size, NULL);
+      g_free(buffer);
+
+
+      wd = lookup_widget(main->aboutDialog, "textviewChangelog");
+      gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(wd), GTK_WRAP_WORD);
+      text = gtk_text_view_get_buffer(GTK_TEXT_VIEW(wd));
+      
+      /* Parse here the XML to get only the relevant entries. */
+      setChangelog(utf8Buffer, size, text);
+
+      g_free(utf8Buffer);
+    }  
+
   chemin = g_build_filename(V_SIM_LEGAL_DIR, V_SIM_AUTHORS_FILE, NULL);
   buffer = (gchar*)0;
   err = (GError*)0;
@@ -310,7 +335,7 @@ static void buildPluginsTab(GtkMain *main)
   gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
 
   renderer = gtk_cell_renderer_text_new();
-  g_object_set(renderer, "xalign", 0.5, NULL);
+  g_object_set(renderer, "xalign", 0., NULL);
   column = gtk_tree_view_column_new_with_attributes(_("Description"), renderer,
 						    "markup", GTK_PLUGINS_COLUMN_DESCRIPTION,
 						    NULL);
@@ -329,3 +354,197 @@ static void buildPluginsTab(GtkMain *main)
   wd = lookup_widget(main->aboutDialog, "scrolledwindowPlugins");
   gtk_container_add(GTK_CONTAINER(wd), tree);
 }
+
+
+/****************************/
+/* XML files for changelog. */
+/****************************/
+
+/*
+  <ChangeLog logiciel="V_Sim" lang="en">
+    <milestone version="3.5" date="(to come)">
+      <intro>All changes leading to 3.5.x series.</intro>
+
+      <entry titre="Development version 3.4.99-3">
+      <li type="capability">Implement a visualisation for the
+      difference</li>
+    </milestone>
+  </ChangeLog>
+*/
+
+/* Known elements. */
+#define PICK_PARSER_ELEMENT_CHANGELOG  "ChangeLog"
+#define PICK_PARSER_ELEMENT_MILESTONE  "milestone"
+#define PICK_PARSER_ELEMENT_ENTRY      "entry"
+#define PICK_PARSER_ELEMENT_LI         "li"
+/* Known attributes. */
+#define PICK_PARSER_ATTRIBUTES_VERSION "version"
+#define PICK_PARSER_ATTRIBUTES_TITRE   "titre"
+#define PICK_PARSER_ATTRIBUTES_TYPE    "type"
+
+static gboolean startAbout, startLi;
+
+/* This method is called for every element that is parsed.
+   The user_data must be the text buffer to write into. */
+static void aboutXML_element(GMarkupParseContext *context _U_,
+			     const gchar         *element_name,
+			     const gchar        **attribute_names,
+			     const gchar        **attribute_values,
+			     gpointer             user_data,
+			     GError             **error _U_)
+{
+  GtkTextBuffer *text;
+  GtkTextIter iter;
+  gchar **version, **refVersion;
+  int minor, refMinor, i;
+
+  g_return_if_fail(user_data);
+  text = GTK_TEXT_BUFFER(user_data);
+
+  DBG_fprintf(stderr, "ChangeLog parser: found '%s' element.\n", element_name);
+  if (!strcmp(element_name, PICK_PARSER_ELEMENT_MILESTONE))
+    {
+      startAbout = FALSE;
+      /* Should have 1 mandatory attribute. */
+      for (i = 0; attribute_names[i]; i++)
+	{
+	  if (!strcmp(attribute_names[i], PICK_PARSER_ATTRIBUTES_VERSION))
+	    {
+	      refVersion = g_strsplit_set(V_SIM_VERSION, ".-", 4);
+	      version    = g_strsplit_set(attribute_values[i], ".", 3);
+	      refMinor   = (atoi(refVersion[2]) == 99)?
+		atoi(refVersion[1]) + 1:atoi(refVersion[1]);
+	      minor      = (version[1])?atoi(version[1]):-1;
+	      startAbout = (minor == refMinor);
+	      g_strfreev(version);
+	      g_strfreev(refVersion);
+	    }
+	}
+    }
+  else if (!strcmp(element_name, PICK_PARSER_ELEMENT_ENTRY))
+    {
+      if (startAbout)
+	{
+	  /* Should have 1 mandatory attribute. */
+	  for (i = 0; attribute_names[i]; i++)
+	    {
+	      if (!strcmp(attribute_names[i], PICK_PARSER_ATTRIBUTES_TITRE))
+		{
+		  gtk_text_buffer_get_end_iter(text, &iter);
+		  gtk_text_buffer_insert_with_tags_by_name(text, &iter,
+							   attribute_values[i],
+							   -1, "bold", NULL);
+		  gtk_text_buffer_get_end_iter(text, &iter);
+		  gtk_text_buffer_insert(text, &iter, ":\n\n", -1);
+		}
+	    }
+	}
+    }
+  else if (!strcmp(element_name, PICK_PARSER_ELEMENT_LI))
+    {
+      if (startAbout)
+	{
+	  /* Should have 1 mandatory attribute. */
+	  for (i = 0; attribute_names[i]; i++)
+	    {
+	      if (!strcmp(attribute_names[i], PICK_PARSER_ATTRIBUTES_TYPE))
+		{
+		  gtk_text_buffer_get_end_iter(text, &iter);
+		  gtk_text_buffer_insert_with_tags_by_name(text, &iter,
+							   attribute_values[i],
+							   -1, "italic", NULL);
+		  gtk_text_buffer_get_end_iter(text, &iter);
+		  gtk_text_buffer_insert(text, &iter, ": ", -1);
+		  startLi = TRUE;
+		}
+	    }
+	}
+    }
+}
+
+/* Check when a element is closed that everything required has been set. */
+static void aboutXML_end(GMarkupParseContext *context _U_,
+			 const gchar         *element_name,
+			 gpointer             user_data,
+			 GError             **error _U_)
+{
+  GtkTextBuffer *text;
+  GtkTextIter iter;
+
+  g_return_if_fail(user_data);
+  text = GTK_TEXT_BUFFER(user_data);
+
+  if (!strcmp(element_name, PICK_PARSER_ELEMENT_MILESTONE))
+    startAbout = FALSE;
+  else if (!strcmp(element_name, PICK_PARSER_ELEMENT_ENTRY))
+    {
+      gtk_text_buffer_get_end_iter(text, &iter);
+      gtk_text_buffer_insert(text, &iter, "\n", -1);
+    }
+  else if (!strcmp(element_name, PICK_PARSER_ELEMENT_LI))
+    {
+      startLi = FALSE;
+      gtk_text_buffer_get_end_iter(text, &iter);
+      gtk_text_buffer_insert(text, &iter, "\n", -1);
+    }
+}
+
+static void aboutXML_text(GMarkupParseContext *context _U_,
+                          const gchar         *buf,
+                          gsize                text_len,  
+                          gpointer             user_data,
+                          GError             **error _U_)
+{
+  GtkTextBuffer *text;
+  GtkTextIter iter;
+
+  g_return_if_fail(user_data);
+  text = GTK_TEXT_BUFFER(user_data);
+
+  if (startAbout && startLi)
+    {
+      gtk_text_buffer_get_end_iter(text, &iter);
+      gtk_text_buffer_insert(text, &iter, buf, text_len);
+      gtk_text_buffer_get_end_iter(text, &iter);
+      gtk_text_buffer_insert(text, &iter, "\n", -1);
+    }
+}
+
+static void setChangelog(const gchar* buffer, gssize size, GtkTextBuffer *text)
+{
+  GMarkupParseContext* xmlContext;
+  GMarkupParser parser;
+  GtkTextTag *tag;
+  gboolean status;
+  GError *error;
+
+
+  /* Create context. */
+  parser.start_element = aboutXML_element;
+  parser.end_element   = aboutXML_end;
+  parser.text          = aboutXML_text;
+  parser.passthrough   = NULL;
+  parser.error         = NULL;
+  xmlContext = g_markup_parse_context_new(&parser, 0, text, NULL);
+
+  tag = gtk_text_buffer_create_tag(text, "bold",
+				   "weight", PANGO_WEIGHT_BOLD, NULL);
+  tag = gtk_text_buffer_create_tag(text, "italic",
+				   "style", PANGO_STYLE_ITALIC, NULL);
+
+  /* Parse data. */
+  startAbout = FALSE;
+  startLi    = FALSE;
+  error = (GError*)0;
+  status = g_markup_parse_context_parse(xmlContext, buffer, size, &error);
+
+  /* Free buffers. */
+  g_markup_parse_context_free(xmlContext);
+
+  if (!status && error)
+    g_warning(error->message);
+
+  if (error)
+    g_error_free(error);
+}
+
