@@ -67,12 +67,34 @@ static void onAddNodeClicked(GtkButton *button, gpointer user_data);
 static void onMoveMethod(GtkToggleButton *toggle, gpointer data);
 static void onPageEnter(GtkNotebook *notebook, GtkNotebookPage *page,
 			guint page_num, gpointer user_data);
+static void onCheckDuplicate(GtkToggleButton *button, gpointer data);
+static void onDuplicate(GtkButton *button, gpointer data);
+static void updateInfos(VisuInteractive *inter,
+			PickMesure *pickMesure, gpointer data);
+static void onMoveToggled(GtkToggleButton *toggle, gpointer data);
+static void onSpinBasis(GtkSpinButton *spin, gpointer data);
+static gboolean removeHighlight(gpointer data);
+static void onChooseBasis(GtkToggleButton *toggle, gpointer data);
+static void onBasisSelected(VisuInteractive *inter, PickMesure *pick, gpointer data);
+static void onApplyBasis(GtkButton *button, gpointer data);
+static void onPopulationChanged(VisuData *dataObj, gint *nodes, gpointer data);
+
+#define GTK_MOVE_INFO				\
+  _("left-button\t\t: drag node(s) in the screen plane\n"	\
+    "shift-left-button\t: drag node(s) along x or y axis\n"	\
+    "control-left-button\t: drag node(s) along z axis\n"	\
+    "right-button\t\t: switch to observe")
 
 /* Pick informations hook */
 static int movedNode = -1;
 static float moveNodeOrigin[3];
 #define GTK_MOVE_NO_NODE         _("<i>none</i>")
 #define GTK_MOVE_NO_REGION       _("<i>unset</i>")
+static gulong onMove_id;
+static gulong onSelection_id;
+static gulong onSpin_id[4];
+static VisuInteractiveId currentMode = interactive_move;
+static guint currentAxe;
 
 /* Widgets */
 static GtkWidget *notebookAction;
@@ -89,28 +111,43 @@ enum
     COLUMN_POINTER_TO_DATA,  /* Pointer to the VisuElement. */
     N_COLUMNS
   };
+static GtkWidget *checkDuplicate, *comboDuplicate, *buttonDupplicate;
 static GtkWidget *labelOriginX, *labelOriginY, *labelOriginZ;
 static GtkWidget *labelScreenHorizontal, *labelScreenVertical;
+static GtkWidget *spinABC[4], *toggleABC[4], *applyBasis;
+static gchar *lblSpinABC[4] = {"O:", "A:", "B:", "C:"};
+static gint prevBasis[4] = {0, 0, 0, 0};
+static guint timeoutBasis[4];
 
 
 /********************/
 /* Public routines. */
 /********************/
-void gtkMoveBuild_interface(GtkMain *main)
+GtkWidget* gtkMoveBuild_interface(GtkMain *main, gchar **label,
+				  gchar **help, GtkWidget **radio)
 {
-  GtkWidget *wd;
+  GtkWidget *wd, *hbox, *vbox;
   GtkCellRenderer *renderer;
   RenderingWindow *window;
   VisuData *data;
   GtkTreeIter iter;
   VisuDataIter dataIter;
+  int i;
+
+  *label = g_strdup("Pick");
+  *help  = g_strdup(GTK_MOVE_INFO);
+  *radio = lookup_widget(main->interactiveDialog, "radioMove");
+  g_signal_connect(G_OBJECT(*radio), "toggled",
+		   G_CALLBACK(onMoveToggled), (gpointer)main->interactiveDialog);
 
   window = visuRenderingWindowGet_current();
-  g_return_if_fail(window);
+  g_return_val_if_fail(window, (GtkWidget*)0);
   data = renderingWindowGet_visuData(window);
-  g_return_if_fail(data);
+  g_return_val_if_fail(data, (GtkWidget*)0);
 
   observeWindow = main->interactiveDialog;
+  
+  vbox = lookup_widget(main->interactiveDialog, "vbox21");
 
   notebookAction = lookup_widget(main->interactiveDialog, "notebookAction");
   g_signal_connect(G_OBJECT(notebookAction), "switch-page",
@@ -187,10 +224,80 @@ void gtkMoveBuild_interface(GtkMain *main)
     }
   gtk_combo_box_set_active(GTK_COMBO_BOX(comboElements), 0);
 
+  /* The duplicate line. */
+  hbox = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
+  wd = gtk_label_new(_("<b>Duplicate nodes:</b>"));
+  gtk_label_set_use_markup(GTK_LABEL(wd), TRUE);
+  gtk_misc_set_alignment(GTK_MISC(wd), 0., 0.5);
+  gtk_box_pack_start(GTK_BOX(hbox), wd, TRUE, TRUE, 0);
+  wd = gtk_label_new(_("<span size=\"smaller\">(the nodes listed in the pick tab)</span>"));
+  gtk_label_set_use_markup(GTK_LABEL(wd), TRUE);
+  gtk_box_pack_start(GTK_BOX(hbox), wd, FALSE, FALSE, 0);
+  gtk_widget_show_all(hbox);
+  hbox = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+  wd = gtk_check_button_new_with_mnemonic(_("du_plicate nodes as they are"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wd), TRUE);
+  gtk_box_pack_start(GTK_BOX(hbox), wd, FALSE, FALSE, 0);
+  checkDuplicate = wd;
+  wd = gtk_label_new(_(" or as new: "));
+  gtk_label_set_use_markup(GTK_LABEL(wd), TRUE);
+  gtk_box_pack_start(GTK_BOX(hbox), wd, FALSE, FALSE, 0);
+  wd = gtk_combo_box_new_with_model(GTK_TREE_MODEL(listElements));
+  renderer = gtk_cell_renderer_text_new();
+  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(wd), renderer, FALSE);
+  gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(wd),
+				renderer, "text", COLUMN_NAME);
+  gtk_combo_box_set_active(GTK_COMBO_BOX(wd), 0);
+  gtk_widget_set_sensitive(wd, FALSE);
+  comboDuplicate = wd;
+  gtk_box_pack_start(GTK_BOX(hbox), wd, FALSE, FALSE, 0);
+  wd = gtk_button_new_with_mnemonic(_("_duplicate"));
+  gtk_box_pack_end(GTK_BOX(hbox), wd, FALSE, FALSE, 5);
+  buttonDupplicate = wd;
+  gtk_widget_show_all(hbox);
+
+  /* The Basis line. */
+  wd = gtk_label_new(_("<b>Change the basis set:</b>"));
+  gtk_widget_show(wd);
+  gtk_label_set_use_markup(GTK_LABEL(wd), TRUE);
+  gtk_misc_set_alignment(GTK_MISC(wd), 0., 0.5);
+  gtk_box_pack_start(GTK_BOX(vbox), wd, FALSE, FALSE, 5);
+  hbox = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+  for (i = 0; i < 4; i++)
+    {
+      wd = gtk_label_new(_(lblSpinABC[i]));
+      gtk_misc_set_alignment(GTK_MISC(wd), 1., 0.5);
+      gtk_box_pack_start(GTK_BOX(hbox), wd, TRUE, TRUE, 5);
+      spinABC[i] = gtk_spin_button_new_with_range(0, dataIter.idMax, 1);
+      onSpin_id[i] = g_signal_connect(G_OBJECT(spinABC[i]), "value-changed",
+				      G_CALLBACK(onSpinBasis), GINT_TO_POINTER(i));
+      gtk_box_pack_start(GTK_BOX(hbox), spinABC[i], FALSE, FALSE, 0);
+      toggleABC[i] = gtk_toggle_button_new();
+      gtk_box_pack_start(GTK_BOX(hbox), toggleABC[i], FALSE, FALSE, 0);
+      gtk_container_add(GTK_CONTAINER(toggleABC[i]),
+			gtk_image_new_from_stock(GTK_STOCK_FIND,
+						 GTK_ICON_SIZE_MENU));
+      g_signal_connect(G_OBJECT(toggleABC[i]), "toggled",
+		       G_CALLBACK(onChooseBasis), GINT_TO_POINTER(i));
+    }
+  applyBasis = gtk_button_new();
+  gtk_box_pack_start(GTK_BOX(hbox), applyBasis, FALSE, FALSE, 5);
+  gtk_container_add(GTK_CONTAINER(applyBasis),
+		    gtk_image_new_from_stock(GTK_STOCK_APPLY, GTK_ICON_SIZE_MENU));
+  gtk_widget_set_sensitive(applyBasis, FALSE);
+  g_signal_connect(G_OBJECT(applyBasis), "clicked",
+		   G_CALLBACK(onApplyBasis), (gpointer)0);
+  gtk_widget_show_all(hbox);
+
+
   g_signal_connect(VISU_INSTANCE, "dataReadyForRendering",
 		   G_CALLBACK(onDataReady), (gpointer)0);
   g_signal_connect(VISU_INSTANCE, "dataNew",
 		   G_CALLBACK(onNewData), (gpointer)0);
+  onNewData((GObject*)0, data, (gpointer)0);
 
   g_signal_connect(G_OBJECT(entryMoveXYZ[0]), "value-changed",
 		   G_CALLBACK(onMovePositionChanged), GINT_TO_POINTER(0));
@@ -213,6 +320,13 @@ void gtkMoveBuild_interface(GtkMain *main)
   wd = lookup_widget(main->interactiveDialog, "radioMoveRegion");
   g_signal_connect(G_OBJECT(wd), "toggled",
 		   G_CALLBACK(onMoveMethod), GINT_TO_POINTER(1));
+
+  g_signal_connect(G_OBJECT(checkDuplicate), "toggled",
+		   G_CALLBACK(onCheckDuplicate), comboDuplicate);
+  g_signal_connect(G_OBJECT(buttonDupplicate), "clicked",
+		   G_CALLBACK(onDuplicate), (gpointer)0);
+
+  return (GtkWidget*)0;
 }
 
 
@@ -243,6 +357,7 @@ static void setLabelsOrigin(GList *nodes)
     }
   else if (nodes)
     {
+      movedNode = -1;
       gtk_label_set_markup(GTK_LABEL(labelMoveId), GTK_MOVE_NO_NODE);
       sprintf(numero, _("%d node(s)"), g_list_length(nodes));
       gtk_label_set_markup(GTK_LABEL(labelNMoves), numero);
@@ -253,6 +368,7 @@ static void setLabelsOrigin(GList *nodes)
     }
   else
     {
+      movedNode = -1;
       gtk_label_set_markup(GTK_LABEL(labelMoveId), GTK_MOVE_NO_NODE);
       gtk_label_set_markup(GTK_LABEL(labelNMoves), GTK_MOVE_NO_REGION);
       numero[0] = '\0';
@@ -294,39 +410,74 @@ static void updateLabelsPosition(GList *nodes, float drag[3])
     }
 }
 
-void gtkMoveUpdate_informations(PickMesure *pickMesure)
+void gtkMoveStart(RenderingWindow *window)
+{
+  VisuInteractive *inter;
+
+  inter = renderingWindowGet_interactive(window);
+
+  DBG_fprintf(stderr, "Gtk Move: set the two signals.\n");
+  visuInteractiveSet_type(inter, currentMode);
+  onMove_id = g_signal_connect(G_OBJECT(inter), "move", G_CALLBACK(updateInfos),
+			       (gpointer)renderingWindowGet_visuData(window));
+  if (currentMode != interactive_move)
+    g_signal_handler_block(G_OBJECT(inter), onMove_id);
+  onSelection_id = g_signal_connect(G_OBJECT(inter), "selection",
+				    G_CALLBACK(onBasisSelected),
+				    (gpointer)renderingWindowGet_visuData(window));
+  if (currentMode != interactive_pick)
+    g_signal_handler_block(G_OBJECT(inter), onSelection_id);
+
+  if (currentMode == interactive_pick)
+    renderingWindowPush_message(window, _("Pick a node with the mouse"));
+}
+void gtkMoveStop(RenderingWindow *window)
+{
+  DBG_fprintf(stderr, "Gtk Move: unset the two signals.\n");
+
+  g_signal_handler_disconnect(G_OBJECT(renderingWindowGet_interactive(window)),
+			      onMove_id);
+  g_signal_handler_disconnect(G_OBJECT(renderingWindowGet_interactive(window)),
+			      onSelection_id);
+  visuInteractiveSet_type(renderingWindowGet_interactive(window),
+			  interactive_none);
+
+  if (currentMode == interactive_pick)
+    renderingWindowPop_message(window);
+}
+static void updateInfos(VisuInteractive *inter,
+			PickMesure *pickMesure, gpointer data)
 {
   PickMesureType type;
-  VisuInteractive *inter;
   GList *nodes, *lst, *tmpLst;
   float *drag;
   VisuNode *node;
-  VisuData *data;
-
-  if (!pickMesureGet_newsAvailable(pickMesure, &type))
-    return;
+  VisuData *dataObj;
 
   DBG_fprintf(stderr, "Gtk Move: callback on move action.\n");
 
+  if (!pickMesureGet_newsAvailable(pickMesure, &type))
+    return /* FALSE */;
+
   nodes = (GList*)0;
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radioMovePick)))
     {
       node = pickMesureGet_selectedNode(pickMesure);
       if (!node)
-	return;
+	return /* FALSE */;
       nodes = g_list_append(nodes, (gpointer)node);
       movedNode = node->number;
     }
   else
     {
-      data = renderingWindowGet_visuData(RENDERING_WINDOW(currentRenderingWindow));
-      g_return_if_fail(data);
+      dataObj = VISU_DATA(data);
+      g_return_if_fail(dataObj);
 
       movedNode = -1;
       tmpLst = lst = gtkPickGet_nodeSelection();
       while(tmpLst)
 	{
-	  node = visuDataGet_nodeFromNumber(data, GPOINTER_TO_INT(tmpLst->data));
+	  node = visuDataGet_nodeFromNumber(dataObj, GPOINTER_TO_INT(tmpLst->data));
 	  if (node)
 	    nodes = g_list_prepend(nodes, (gpointer)node);
 	  tmpLst = g_list_next(tmpLst);
@@ -334,28 +485,28 @@ void gtkMoveUpdate_informations(PickMesure *pickMesure)
       g_list_free(lst);
 
       if (!nodes)
-	return;
+	return /* FALSE */;
     }
-  inter = (VisuInteractive*)g_object_get_data(G_OBJECT(observeWindow),
-					      "visuInteractive_data");
   switch (type)
     {
     case PICK_DRAG_START:
-      openGLInteractiveStart_move(inter, nodes);
+      visuInteractiveStart_move(inter, nodes);
       setLabelsOrigin(nodes);
       break;
     case PICK_DRAG_STOP:
-      openGLInteractiveStop_move(inter, nodes);
+      visuInteractiveStop_move(inter, nodes);
       break;
     case PICK_DRAG_MOVE:
       drag = pickMesureGet_drag(pickMesure);
-      openGLInteractiveMove(inter, nodes, drag);
+      visuInteractiveMove(inter, nodes, drag);
       updateLabelsPosition(nodes, drag);
       break;
     default:
       g_error("Not a drag action!");
     }
   g_list_free(nodes);
+
+  return /* FALSE */;
 }
 
 /****************/
@@ -381,12 +532,17 @@ static void onNewData(GObject *obj _U_, VisuData *dataObj, gpointer data _U_)
 {
   g_signal_connect(G_OBJECT(dataObj), "OpenGLThetaPhiOmega",
 		   G_CALLBACK(refreshMoveAxesValues), (gpointer)0);
+  g_signal_connect(G_OBJECT(dataObj), "NodePopulationDecrease",
+		   G_CALLBACK(onPopulationChanged), (gpointer)0);
+  g_signal_connect(G_OBJECT(dataObj), "NodePopulationIncrease",
+		   G_CALLBACK(onPopulationChanged), (gpointer)0);
 }
 
 static void onDataReady(GObject *obj _U_, VisuData *dataObj, gpointer data _U_)
 {
   GtkTreeIter iter;
   VisuDataIter dataIter;
+  int i;
 
   if (dataObj)
     {
@@ -408,6 +564,9 @@ static void onDataReady(GObject *obj _U_, VisuData *dataObj, gpointer data _U_)
 	    }
 	  gtk_combo_box_set_active(GTK_COMBO_BOX(comboElements), 0);
 	}
+      for (i = 0; i < 4; i++)
+	gtk_spin_button_set_range(GTK_SPIN_BUTTON(spinABC[i]), 0,
+				  dataIter.idMax);
     }
   else
     {
@@ -558,7 +717,7 @@ static void onRemoveNodeClicked(GtkButton *button _U_, gpointer user_data _U_)
   int *nodes;
   VisuData *dataObj;
   VisuNode *node;
-  int iEle, i;
+  int iEle, i, n;
   GList *lst, *tmpLst;
 
   DBG_fprintf(stderr, "Gtk Observe/pick: remove the selected node %d.\n", movedNode);
@@ -592,19 +751,25 @@ static void onRemoveNodeClicked(GtkButton *button _U_, gpointer user_data _U_)
     }
   else
     {
+      DBG_fprintf(stderr, "Gtk Move: get list of selected nodes.\n");
       tmpLst = lst = gtkPickGet_nodeSelection();
-      nodes = g_malloc(sizeof(int) * (g_list_length(lst) + 1));
-      i = 0;
-      while(tmpLst)
+      n = g_list_length(lst);
+      if (n > 0)
 	{
-	  nodes[i] = GPOINTER_TO_INT(tmpLst->data);
-	  i += 1;
-	  tmpLst = g_list_next(tmpLst);
+	  nodes = g_malloc(sizeof(int) * (n + 1));
+	  i = 0;
+	  while(tmpLst)
+	    {
+	      nodes[i] = GPOINTER_TO_INT(tmpLst->data);
+	      DBG_fprintf(stderr, " | will remove %3d %d\n", i, nodes[i]);
+	      i += 1;
+	      tmpLst = g_list_next(tmpLst);
+	    }
+	  nodes[i] = -1;
+	  visuDataRemove_nodes(dataObj, nodes);
+	  g_free(nodes);
 	}
       g_list_free(lst);
-      nodes[i] = -1;
-      visuDataRemove_nodes(dataObj, nodes);
-      g_free(nodes);
 
       visuData_createAllNodes(dataObj);
     }
@@ -621,7 +786,6 @@ static void onAddNodeClicked(GtkButton *button _U_, gpointer user_data _U_)
   gboolean valid;
   VisuElement *element;
   float xyz[3];
-  int i;
 
   window = RENDERING_WINDOW(visuRenderingWindowGet_current());
   g_return_if_fail(window);
@@ -634,12 +798,230 @@ static void onAddNodeClicked(GtkButton *button _U_, gpointer user_data _U_)
   gtk_tree_model_get(GTK_TREE_MODEL(listElements), &iter,
 		     COLUMN_POINTER_TO_DATA, &element,
 		     -1);
-  for (i = 0; i < 3; i++)
-    xyz[i] = (float)numericalEntryGet_value(NUMERICAL_ENTRY(entryAddXYZ[i]));
-  visuDataAdd_nodeFromElement(dataObj, element, xyz);
+  xyz[0] = (float)numericalEntryGet_value(NUMERICAL_ENTRY(entryAddXYZ[0]));
+  xyz[1] = (float)numericalEntryGet_value(NUMERICAL_ENTRY(entryAddXYZ[1]));
+  xyz[2] = (float)numericalEntryGet_value(NUMERICAL_ENTRY(entryAddXYZ[2]));
+  visuDataAdd_nodeFromElement(dataObj, element, xyz, TRUE);
 
   /* We update all drawings. */
   visuDataEmit_nodePositionChanged(dataObj);
   visuData_createNodes(dataObj, element);
   g_idle_add(visuObjectRedraw, (gpointer)0);
 }
+static void onCheckDuplicate(GtkToggleButton *button, gpointer data)
+{
+  gtk_widget_set_sensitive(GTK_WIDGET(data), !gtk_toggle_button_get_active(button));
+}
+static void onDuplicate(GtkButton *button _U_, gpointer data _U_)
+{
+  RenderingWindow *window;
+  VisuData *dataObj;
+  GList *tmpLst, *lst;
+  GtkTreeIter iter;
+  gboolean valid, custom;
+  VisuElement *element;
+  float coord[3];
+  VisuNode *node;
+
+  DBG_fprintf(stderr, "Gtk Move: duplicate selected nodes.\n");
+
+  window = RENDERING_WINDOW(visuRenderingWindowGet_current());
+  g_return_if_fail(window);
+
+  dataObj = renderingWindowGet_visuData(window);
+
+  custom = !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkDuplicate));
+  if (custom)
+    {
+      valid = gtk_combo_box_get_active_iter(GTK_COMBO_BOX(comboDuplicate), &iter);
+      g_return_if_fail(valid);
+
+      gtk_tree_model_get(GTK_TREE_MODEL(listElements), &iter,
+			 COLUMN_POINTER_TO_DATA, &element,
+			 -1);
+      DBG_fprintf(stderr, "Gtk Move: change element to '%s'.\n", element->name);
+    }
+
+  lst = gtkPickGet_nodeSelection();
+  if (!lst)
+    return;
+
+  for (tmpLst = lst; tmpLst; tmpLst = g_list_next(tmpLst))
+    {
+      node = visuDataGet_nodeFromNumber
+	(dataObj, (guint)GPOINTER_TO_INT(tmpLst->data));
+      visuDataGet_nodePosition(dataObj, node, coord);
+      if (custom)
+	visuDataAdd_nodeFromElement(dataObj, element, coord, !tmpLst->next);
+      else
+	visuDataAdd_nodeFromIndex(dataObj, node->posElement, coord, !tmpLst->next);
+    }
+  g_list_free(lst);
+
+  /* We update all drawings. */
+  if (custom)
+    visuData_createNodes(dataObj, element);
+  else
+    visuData_createAllNodes(dataObj);
+  g_idle_add(visuObjectRedraw, (gpointer)0);
+}
+static void onMoveToggled(GtkToggleButton *toggle, gpointer data)
+{
+  GtkWidget *wd;
+  gboolean value;
+
+  value = gtk_toggle_button_get_active(toggle);
+  wd = lookup_widget(GTK_WIDGET(data), "alignment41");
+  gtk_widget_set_sensitive(wd, value);
+}
+static void onSpinBasis(GtkSpinButton *spin, gpointer data)
+{
+  int i;
+  RenderingWindow *window;
+  PickMesure *pick;
+  gboolean valid;
+  VisuData *dataObj;
+  
+  i = GPOINTER_TO_INT(data);
+
+  window = RENDERING_WINDOW(currentRenderingWindow);
+  pick = visuInteractiveGet_pickMesure(renderingWindowGet_interactive(window));
+  dataObj = renderingWindowGet_visuData(window);
+
+  /* Remove the previous one. */
+  if (prevBasis[i] > 0)
+    pickMesureSet_highlight(pick, (guint)(prevBasis[i] - 1), FALSE);
+  if (timeoutBasis[i] > 0)
+    g_source_remove(timeoutBasis[i]);
+
+  prevBasis[i] = (gint)gtk_spin_button_get_value(spin);
+  if (prevBasis[i] > 0 && visuDataGet_nodeFromNumber(dataObj, prevBasis[i]))
+    {
+      /* Add a new highlight. */
+#if GLIB_MINOR_VERSION > 13
+      timeoutBasis[i] = g_timeout_add_seconds(1, removeHighlight,
+					      GINT_TO_POINTER(prevBasis[i]));
+#else
+      timeoutBasis[i] = g_timeout_add(1000, removeHighlight,
+				      GINT_TO_POINTER(prevBasis[i]));
+#endif
+      pickMesureSet_highlight(pick, (guint)(prevBasis[i] - 1), TRUE);
+    }
+
+  valid = TRUE;
+  for(i = 0; i < 4; i++)
+    valid = valid && (gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinABC[i])) > 0.);
+  gtk_widget_set_sensitive(applyBasis, valid);
+  
+  pickMesureRebuild_classicalList(dataObj);
+  g_idle_add(visuObjectRedraw, GINT_TO_POINTER(TRUE));
+}
+static gboolean removeHighlight(gpointer data)
+{
+  int i;
+  RenderingWindow *window;
+  PickMesure *pick;
+
+  i = GPOINTER_TO_INT(data);
+  g_return_val_if_fail(i > 0, FALSE);
+
+  window = RENDERING_WINDOW(currentRenderingWindow);
+  pick = visuInteractiveGet_pickMesure(renderingWindowGet_interactive(window));
+
+  /* Remove the previous one. */
+  pickMesureSet_highlight(pick, (guint)(i - 1), FALSE);
+  pickMesureRebuild_classicalList(renderingWindowGet_visuData(window));
+  g_idle_add(visuObjectRedraw, GINT_TO_POINTER(TRUE));
+
+  return FALSE;
+}
+static void onChooseBasis(GtkToggleButton *toggle, gpointer data)
+{
+  guint i;
+  VisuInteractive *inter;
+  PickMesure *pick;
+  gboolean valid;
+
+  inter = renderingWindowGet_interactive(RENDERING_WINDOW(currentRenderingWindow));
+
+  DBG_fprintf(stderr, "Gtk Move: one toggle chooser has been toggled.\n");
+  valid = gtk_toggle_button_get_active(toggle);
+  if (valid)
+    {
+      currentAxe = GPOINTER_TO_INT(data);
+      for(i = 0; i < 4; i++)
+	if (i != currentAxe)
+	  gtk_widget_set_sensitive(toggleABC[i], FALSE);
+      DBG_fprintf(stderr, "Gtk Move: change the mode to pick.\n");
+      currentMode = interactive_pick;
+      pick = visuInteractiveGet_pickMesure(inter);
+      pickMesureSet_pickNode(pick, -1, PICK_REFERENCE_1);
+      pickMesureSet_pickNode(pick, -1, PICK_REFERENCE_2);
+      g_signal_handler_block(G_OBJECT(inter), onMove_id);
+      g_signal_handler_unblock(G_OBJECT(inter), onSelection_id);
+
+      renderingWindowPush_message(RENDERING_WINDOW(currentRenderingWindow),
+				  _("Pick a node with the mouse"));
+    }
+  else
+    {
+      for(i = 0; i < 4; i++)
+	gtk_widget_set_sensitive(toggleABC[i], TRUE);
+      DBG_fprintf(stderr, "Gtk Move: change the mode to move.\n");
+      currentMode = interactive_move;
+      g_signal_handler_unblock(G_OBJECT(inter), onMove_id);
+      g_signal_handler_block(G_OBJECT(inter), onSelection_id);
+
+      renderingWindowPop_message(RENDERING_WINDOW(currentRenderingWindow));
+    }
+  visuInteractiveSet_type(inter, currentMode);
+  gtk_widget_set_sensitive(lookup_widget(observeWindow, "alignment43"), !valid);
+  gtk_widget_set_sensitive(lookup_widget(observeWindow, "alignment44"), !valid);
+  gtk_widget_set_sensitive(lookup_widget(observeWindow, "alignment27"), !valid);
+}
+static void onBasisSelected(VisuInteractive *inter _U_,
+			    PickMesure *pick, gpointer data _U_)
+{
+  VisuNode *node;
+
+  node = pickMesureGet_selectedNode(pick);
+  if (node)
+    {
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinABC[currentAxe]),
+				node->number + 1);
+      gtk_toggle_button_set_active
+	(GTK_TOGGLE_BUTTON(toggleABC[currentAxe]), FALSE);
+    }
+}
+static void onApplyBasis(GtkButton *button _U_, gpointer data _U_)
+{
+  VisuData *dataObj;
+
+  dataObj = renderingWindowGet_visuData(RENDERING_WINDOW(currentRenderingWindow));
+  visuDataSet_newBasis(dataObj,
+		       (guint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinABC[0])) - 1,
+		       (guint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinABC[1])) - 1,
+		       (guint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinABC[2])) - 1,
+		       (guint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinABC[3])) - 1);
+}
+static void onPopulationChanged(VisuData *dataObj, gint *nodes _U_, gpointer data _U_)
+{
+  VisuDataIter iter;
+
+  visuDataIter_new(dataObj, &iter);
+  g_signal_handler_block(G_OBJECT(spinABC[0]), onSpin_id[0]);
+  gtk_spin_button_set_range(GTK_SPIN_BUTTON(spinABC[0]), 0, iter.idMax);
+  g_signal_handler_unblock(G_OBJECT(spinABC[0]), onSpin_id[0]);
+
+  g_signal_handler_block(G_OBJECT(spinABC[1]), onSpin_id[1]);
+  gtk_spin_button_set_range(GTK_SPIN_BUTTON(spinABC[1]), 0, iter.idMax);
+  g_signal_handler_unblock(G_OBJECT(spinABC[1]), onSpin_id[1]);
+
+  g_signal_handler_block(G_OBJECT(spinABC[2]), onSpin_id[2]);
+  gtk_spin_button_set_range(GTK_SPIN_BUTTON(spinABC[2]), 0, iter.idMax);
+  g_signal_handler_unblock(G_OBJECT(spinABC[2]), onSpin_id[2]);
+
+  g_signal_handler_block(G_OBJECT(spinABC[3]), onSpin_id[3]);
+  gtk_spin_button_set_range(GTK_SPIN_BUTTON(spinABC[3]), 0, iter.idMax);
+  g_signal_handler_unblock(G_OBJECT(spinABC[3]), onSpin_id[3]);
+}
