@@ -44,6 +44,8 @@
 #ifndef GTK_OPENGLWIDGET_H
 #define GTK_OPENGLWIDGET_H
 
+#include "visu_data.h"
+
 /**
  * OpenGLWidget:
  *
@@ -60,12 +62,12 @@ typedef struct _OpenGLWidgetClass OpenGLWidgetClass;
 /**
  * RedrawMethod:
  * @lists: a string array, NULL terminated.
- * @data: some associated data.
+ * @data: the #VisuData to draw.
  *
  * Methods of this prototype are called when the redraw of the OpenGL zone
  * should be done.
  */
-typedef void (*RedrawMethod)(const char** lists, gpointer data);
+typedef void (*RedrawMethod)(const char** lists, VisuData *data);
 
 /**
  * TYPE_OPENGL_WIDGET:
@@ -153,13 +155,13 @@ gboolean openGLWidgetSet_current(OpenGLWidget *render, gboolean force);
  * openGLWidgetSet_redraw:
  * @render: a #OpenGLWidget object ;
  * @method: a redraw method ;
- * @data: a user defined pointer.
+ * @data: the #VisuData to draw.
  *
  * This method is used to defined a redraw method for the OpenGL area. By doing this
  * the area will automatically redraw itself when necessary. Before doing it
  * it calls openGLWidgetSet_current() ; and after it calls openGLWidgetSwap_buffers().
  */
-void openGLWidgetSet_redraw(OpenGLWidget *render, RedrawMethod method, gpointer data);
+void openGLWidgetSet_redraw(OpenGLWidget *render, RedrawMethod method, VisuData *data);
 /**
  * openGLWidgetRedraw:
  * @render: a #OpenGLWidget object.
