@@ -55,11 +55,11 @@
 #include "visu_object.h"
 #include "visu_tools.h"
 #include "gtk_renderingWindowWidget.h"
-
+#include "visu_commandLine.h"
 
 static guint saveResourcesContextId, saveParametersContextId;
 static gchar *lastParsedDir;
-static GtkWidget *checkXMLResources, *checkXMLParameters;
+static GtkWidget *checkXMLResources, *checkXMLParameters, *checkXMLCommandLine;
 
 /* Local Callbacks. */
 static void onLoadResButtonClicked(GtkButton *button, gpointer data);
@@ -141,7 +141,7 @@ void gtkSaveInit()
 
   /* Set initial values. */
   wd = lookup_widget(saveDialog, "filechooserwidgetResources");
-  directory = gtkMainClassGet_lastOpenDirectory();
+  directory = visuGtkGet_lastOpenDirectory();
   if (directory)
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(wd), directory);
   g_signal_connect(G_OBJECT(wd), "selection-changed",
@@ -236,12 +236,21 @@ void gtkSaveInit()
     gtk_check_button_new_with_mnemonic(_("Export all parameter descriptions"
 					 " to an _XML file"));
   gtk_box_pack_start(GTK_BOX(wd), checkXMLParameters, FALSE, FALSE, 0);
-  gtk_container_set_border_width(GTK_CONTAINER(checkXMLParameters), 15);
+  gtk_container_set_border_width(GTK_CONTAINER(checkXMLParameters), 5);
   gtk_widget_set_name(checkXMLParameters, "message_radio");
   gtk_widget_show(checkXMLParameters);
+
+  checkXMLCommandLine =
+    gtk_check_button_new_with_mnemonic(_("Export all command line _options"
+					 " to an XML file"));
+  gtk_box_pack_start(GTK_BOX(wd), checkXMLCommandLine, FALSE, FALSE, 0);
+  gtk_container_set_border_width(GTK_CONTAINER(checkXMLCommandLine), 5);
+  gtk_widget_set_name(checkXMLCommandLine, "message_radio");
+  gtk_widget_show(checkXMLCommandLine);
 #else
   checkXMLResources = (GtkWidget*)0;
   checkXMLParameters = (GtkWidget*)0;
+  checkXMLCommandLine = (GtkWidget*)0;
 #endif
 
   wd = lookup_widget(saveDialog, "vbox18");
@@ -306,7 +315,7 @@ static void onLoadResButtonClicked(GtkButton *button _U_, gpointer data)
 
   filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_selector));
   directory = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(file_selector));
-  gtkMainClassSet_lastOpenDirectory(directory);
+  visuGtkSet_lastOpenDirectory(directory, VISU_DIR_CONF);
   g_free(directory);
 
   dataObj = renderingWindowGet_visuData(currentRenderingWindow);
@@ -331,12 +340,6 @@ static void onLoadResButtonClicked(GtkButton *button _U_, gpointer data)
   g_string_free(message, TRUE);
   g_free(basename);
 
-  DBG_fprintf(stderr, "Gtk Save: emitting 'resourcesLoaded' signal.\n");
-  g_signal_emit(VISU_INSTANCE,
-		VISU_GET_CLASS(VISU_INSTANCE)->resourcesLoaded_signal_id,
-		0, dataObj, NULL);
-  DBG_fprintf(stderr, "Gtk Save: emission done.\n");
-
   if (res)
     g_idle_add(visuObjectRedraw, (gpointer)0);
 }
@@ -411,6 +414,9 @@ static void saveAction(int type, GtkWidget *saveDialog, GtkEntry *entry)
   else if (type == VISU_CONFIGFILE_PARAMETER && checkXMLParameters &&
 	   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkXMLParameters)))
     resOk = visuConfigFileExport_toXML(file, VISU_CONFIGFILE_PARAMETER, &error);
+  else if (type == VISU_CONFIGFILE_PARAMETER && checkXMLCommandLine &&
+	   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkXMLCommandLine)))
+    resOk = commandLineExport(file, &error);
   else
     resOk = visuConfigFileSave(type, file, &lines, dataObj, &error);
   if (!resOk)
