@@ -48,10 +48,11 @@
 #include <visu_tools.h>
 #include <OSOpenGL/visu_openGL.h>
 
-static GLuint BASE; /* for the font display list */
+static GLuint BASE;  /* for the font display list */
+static GLuint SMALL; /* for the small font display list */
 static gboolean textListHaveBeenBuilt = FALSE;
 
-void openGLText_drawChars(gchar *s)
+void openGLText_drawChars(gchar *s, OpenGLTextSize size)
 {
   g_return_if_fail(s);
 
@@ -59,24 +60,41 @@ void openGLText_drawChars(gchar *s)
 
   DBG_fprintf(stderr, "OpenGL Text: put text '%s'.\n", s);
   glPushAttrib(GL_LIST_BIT);
-  glListBase(BASE);
+  switch (size)
+    {
+    case TEXT_NORMAL:
+      glListBase(BASE);
+      break;
+    case TEXT_SMALL:
+      glListBase(SMALL);
+      break;
+    default:
+      glListBase(BASE);
+    }
   glCallLists((int)strlen(s), GL_UNSIGNED_BYTE, (GLubyte *)s);
   glPopAttrib();
 }
 
 void openGLText_initFontList()
 {
-  DBG_fprintf(stderr, "OpenGL Text: creating font list : %d.\n", BASE);
   if (textListHaveBeenBuilt)
     return;
 
-  BASE = visuOpenGLinit_fontList();
-  textListHaveBeenBuilt = TRUE;
+  BASE  = visuOpenGLinit_fontList(18);
+  SMALL = visuOpenGLinit_fontList(14);
+  textListHaveBeenBuilt = (BASE > 0 && SMALL > 0);
+  DBG_fprintf(stderr, "OpenGL Text: creating font list: %d %d.\n", BASE, SMALL);
 }
 
 void openGLText_rebuildFontList()
 {
   DBG_fprintf(stderr, "OpenGL Text: force rebuilding font list.\n");
-  textListHaveBeenBuilt = TRUE;
-  BASE = visuOpenGLinit_fontList();
+  textListHaveBeenBuilt = FALSE;
+  openGLText_initFontList();
+}
+
+void openGLText_onNewContext()
+{
+  DBG_fprintf(stderr, "OpenGL Text: new context, forgetting fonts.\n");
+  textListHaveBeenBuilt = FALSE;
 }
