@@ -44,7 +44,6 @@
 #include "gtk_wire.h"
 
 #include <gtk/gtk.h>
-#include <support.h>
 
 #include <visu_object.h>
 #include <visu_tools.h>
@@ -107,14 +106,16 @@ GtkWidget* gtkPairsWireBuild_widgets()
   GtkWidget *radiobutton2;
   GtkWidget *radiobutton3;
   GtkWidget *buttonApply;
-  GtkTooltips *tooltips;
   GtkWidget *frame;
+#if GTK_MINOR_VERSION < 12
+  GtkTooltips *tooltips;
+  tooltips = gtk_tooltips_new ();
+#endif
 
   DBG_fprintf(stderr, "Gtk Wire : building interface for wire pairs.\n");
 
-  tooltips = gtk_tooltips_new ();
 
-  hbox1 = gtk_hbox_new (FALSE, 0);
+  hbox1 = gtk_hbox_new (FALSE, 5);
 
   vbox4 = gtk_vbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox1), vbox4, FALSE, FALSE, 0);
@@ -139,7 +140,7 @@ GtkWidget* gtkPairsWireBuild_widgets()
   frame = gtk_frame_new("");
 /*   gtk_widget_set_size_request(frame, -1, 75); */
   gtk_widget_show(frame);
-  gtk_box_pack_start (GTK_BOX (hbox1), frame, TRUE, TRUE, 5);
+  gtk_box_pack_start (GTK_BOX (hbox1), frame, TRUE, TRUE, 0);
 
   checkNonLinear = gtk_check_button_new ();
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkNonLinear), getWireNonLinear());
@@ -233,7 +234,7 @@ GtkWidget* gtkPairsWireBuild_widgets()
   buttonApply = gtk_button_new_from_stock ("gtk-apply");
   gtk_widget_show (buttonApply);
   gtk_box_pack_start (GTK_BOX (hbox5), buttonApply, TRUE, FALSE, 0);
-  gtk_tooltips_set_tip (tooltips, buttonApply, _("Click here to aply all changes done to the curves."), NULL);
+  gtk_widget_set_tooltip_text(buttonApply, _("Click here to aply all changes done to the curves."));
 
   /* Curve + label */
   vboxCurve[0] = gtk_vbox_new (FALSE, 0);
