@@ -73,9 +73,9 @@ static gboolean readLinkStipple(gchar **lines, int nbLines, int position,
 static gboolean readWireNonLinear(gchar **lines, int nbLines, int position,
 				  VisuData *dataObj, GError **error);
 /* This function writes the parameter. */
-static gboolean exportParametersWire(GString *data, int *nbLinesWritten, VisuData *dataObj);
+static void exportParametersWire(GString *data, VisuData *dataObj);
 /* This function save the resources. */
-static gboolean exportResourcesWire(GString *data, int *nbLinesWritten, VisuData *dataObj);
+static void exportResourcesWire(GString *data, VisuData *dataObj);
 
 #define FLAG_PARAMETER_WIRE   "wire_nonLinear"
 #define DESC_PARAMETER_WIRE   "If the color of the pairs are corrected by their length ; boolean 0 or 1"
@@ -452,14 +452,10 @@ static gboolean readWireNonLinear(gchar **lines, int nbLines, int position,
   return TRUE;
 }
 /* This function writes the parameter. */
-static gboolean exportParametersWire(GString *data, int *nbLinesWritten,
-				     VisuData *dataObj _U_)
+static void exportParametersWire(GString *data, VisuData *dataObj _U_)
 {
   g_string_append_printf(data, "# %s\n", DESC_PARAMETER_WIRE);
   g_string_append_printf(data, "%s: %i\n\n", FLAG_PARAMETER_WIRE, wireNonLinear);
-  *nbLinesWritten = 3;
-
-  return TRUE;
 }
 void exportPairsWidth(VisuElement *ele1, VisuElement *ele2,
 		      VisuPairData *data, gpointer userData)
@@ -492,7 +488,6 @@ void exportPairsWidth(VisuElement *ele1, VisuElement *ele2,
 			       FLAG_RESOURCES_LINK_WIDTH,
 			       ele1->name, ele2->name, data->minMax[PAIRS_MIN],
 			       data->minMax[PAIRS_MAX], *width);
-      *str->nbLinesWritten += 2;
     }
   stipple = (guint16*)visuPairGet_linkProperty(data, "stipple");
   if (stipple)
@@ -519,23 +514,17 @@ void exportPairsWidth(VisuElement *ele1, VisuElement *ele2,
 			       data->minMax[PAIRS_MAX], *stipple);
     }
 }
-static gboolean exportResourcesWire(GString *data, int *nbLinesWritten,
-				    VisuData *dataObj)
+static void exportResourcesWire(GString *data, VisuData *dataObj)
 {
   struct foreachFuncExport_struct str;
 
   g_string_append_printf(data, "# %s\n", DESC_RESOURCES_WIRE_WIDTH);
   g_string_append_printf(data, "%s:\n    %d\n", FLAG_RESOURCES_WIRE_WIDTH, wireWidth);
-  *nbLinesWritten = 3;
   str.data           = data;
-  str.nbLinesWritten = nbLinesWritten;
   str.dataObj        = dataObj;
   g_string_append_printf(data, "# %s\n", DESC_RESOURCES_LINK_WIDTH);
   visuPairForeach(exportPairsWidth, &str);
   g_string_append_printf(data, "\n");
-  *nbLinesWritten += 2;
-
-  return TRUE;
 }
 
 
