@@ -43,13 +43,16 @@
 */
 #include "panelBox.h"
 
+#include <support.h>
 #include <visu_object.h>
 #include <visu_tools.h>
 #include <visu_data.h>
+#include <visu_basic.h>
 #include <extensions/box.h>
 #include <coreTools/toolColor.h>
 #include <extraGtkFunctions/gtk_stippleComboBoxWidget.h>
 #include <extraGtkFunctions/gtk_lineObjectWidget.h>
+#include <extraFunctions/geometry.h>
 
 /* Local objects. */
 static GtkWidget *panelBox;
@@ -64,6 +67,8 @@ static GtkWidget *imagePeriodic;
 static GtkWidget *labelPeriodic;
 static gulong transId;
 static gboolean transStatus = FALSE;
+static GtkWidget *comboUnit;
+static GtkWidget *checkDiff;
 
 /* Local variables. */
 static gboolean disableCallbacks;
@@ -89,9 +94,11 @@ static void onExpandChanged(GtkSpinButton* button, gpointer data);
 static void onExpandChecked(GtkToggleButton* button, gpointer data);
 static void onElementRenderChanged(VisuData *dataObj,
 				   VisuElement *element, gpointer data);
+static void onNodePositionChanged(VisuData *dataObj, gpointer data);
 static void onStippleChanged(LineObject *line, gint stipple, gpointer data);
 static void onStippleExpandChanged(StippleComboBox *combo, gint stipple, gpointer data);
-
+static void onUnitChanged(GtkComboBox *combo, gpointer data);
+static void onDiffChanged(GtkToggleButton *toggle, gpointer data);
 
 ToolPanel* panelBox_init()
 {
@@ -126,6 +133,9 @@ ToolPanel* panelBox_init()
       spinExpandXYZ[i] = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 2);
       gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spinExpandXYZ[i]), TRUE);
     }
+  checkDiff =
+    gtk_check_button_new_with_mnemonic(_("Visualise the node _displacements"));
+  comboUnit = gtk_combo_box_new_text();
 
   /* Create the callbacks of all the sensitive widgets. */
   g_signal_connect(G_OBJECT(panelBox), "page-entered",
@@ -146,14 +156,17 @@ static GtkWidget *createInteriorBox()
 {
   GtkWidget *vbox, *hbox;
   GtkWidget *label, *align;
-  GtkTooltips *tooltips;
   int i;
 #define X_LABEL _("dx:")
 #define Y_LABEL _("dy:")
 #define Z_LABEL _("dz:")
   char *xyz[3];
+  const gchar **units;
+#if GTK_MINOR_VERSION < 12
+  GtkTooltips *tooltips;
 
   tooltips = gtk_tooltips_new ();
+#endif
 
   vbox = gtk_vbox_new(FALSE, 0);
 
@@ -162,7 +175,7 @@ static GtkWidget *createInteriorBox()
   /*********************/
   lineBox = lineObjectNew(_("Bounding box"));
   align = gtk_alignment_new(0.5, 0., 1, 1);
-  gtk_alignment_set_padding(GTK_ALIGNMENT(align), 0, 25, 0, 0);
+  gtk_alignment_set_padding(GTK_ALIGNMENT(align), 0, 15, 0, 0);
   gtk_container_add(GTK_CONTAINER(align), lineBox);
   gtk_box_pack_start(GTK_BOX(vbox), align, FALSE, FALSE, 0);
   g_signal_connect(G_OBJECT(lineBox), "use-changed",
@@ -190,12 +203,14 @@ static GtkWidget *createInteriorBox()
   gtk_label_set_use_markup(GTK_LABEL(labelPeriodic), TRUE);
   gtk_label_set_line_wrap(GTK_LABEL(labelPeriodic), TRUE);
 
-  gtk_box_pack_start(GTK_BOX(vbox), vboxPeriodic, FALSE, FALSE, 0);
+  align = gtk_alignment_new(0.5, 0., 1, 1);
+  gtk_alignment_set_padding(GTK_ALIGNMENT(align), 0, 15, 0, 0);
+  gtk_container_add(GTK_CONTAINER(align), vboxPeriodic);
+  gtk_box_pack_start(GTK_BOX(vbox), align, FALSE, FALSE, 0);
   /* The translations. */
-  gtk_tooltips_set_tip(tooltips, checkAllowTranslations,
-		       _("Translations are given in box coordinates and nodes are "
-			 "automatically translated back into the bounding box."),
-		       NULL);
+  gtk_widget_set_tooltip_text(checkAllowTranslations,
+			      _("Translations are given in box coordinates and nodes are "
+				"automatically translated back into the bounding box."));
   gtk_box_pack_start(GTK_BOX(vboxPeriodic), checkAllowTranslations, FALSE, FALSE, 0);
   transId = g_signal_connect(G_OBJECT(checkAllowTranslations), "toggled",
 			     G_CALLBACK(onTranslationChecked), (gpointer)0);
@@ -221,11 +236,11 @@ static GtkWidget *createInteriorBox()
   /* The replication. */
   hbox = gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vboxPeriodic), hbox, FALSE, FALSE, 0);
-  gtk_tooltips_set_tip(tooltips, checkAllowExpand,
-		       _("The size of the expansion is given in box coordinates."
-			 " Nodes are automatically translated back into the new"
-			 " defined area. The drawn bounding box is kept to the"
-			 " original size."), NULL);
+  gtk_widget_set_tooltip_text(checkAllowExpand,
+			      _("The size of the expansion is given in box coordinates."
+				" Nodes are automatically translated back into the new"
+				" defined area. The drawn bounding box is kept to the"
+				" original size."));
   gtk_box_pack_start(GTK_BOX(hbox), checkAllowExpand, TRUE, TRUE, 0);
   g_signal_connect(G_OBJECT(checkAllowExpand), "toggled",
 		   G_CALLBACK(onExpandChecked), (gpointer)0);
@@ -256,6 +271,39 @@ static GtkWidget *createInteriorBox()
       gtk_box_pack_start(GTK_BOX(hbox), spinExpandXYZ[i], FALSE, FALSE, 0);
     }
 
+  /***********************/
+  /* The geometry stuff. */
+  /***********************/
+  hbox = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+  label = gtk_label_new(_("<b>Geometry</b>"));
+  gtk_widget_set_name(label, "label_head");
+  gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
+  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 2);
+
+  hbox = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+  label = gtk_label_new(_("Set the unit of the file:"));
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
+
+  gtk_widget_set_sensitive(comboUnit, FALSE);
+  units = visuToolsGet_unitNames();
+  for (i = 0; units[i]; i++)
+    gtk_combo_box_append_text(GTK_COMBO_BOX(comboUnit), units[i]);
+  gtk_combo_box_set_active(GTK_COMBO_BOX(comboUnit), 0);
+  g_signal_connect(G_OBJECT(comboUnit), "changed",
+		   G_CALLBACK(onUnitChanged), (gpointer)0);
+  gtk_box_pack_start(GTK_BOX(hbox), comboUnit, FALSE, FALSE, 0);
+
+  gtk_widget_set_tooltip_text(checkDiff,
+			      _("When a new file is loaded, draw arrows on  nodes that"
+				" represent their displacements with respect to their"
+				" previous positions."));
+  g_signal_connect(G_OBJECT(checkDiff), "toggled",
+		   G_CALLBACK(onDiffChanged), (gpointer)0);
+  gtk_box_pack_start(GTK_BOX(vbox), checkDiff, FALSE, FALSE, 0);
+
   gtk_widget_show_all(vbox);
   return vbox;
 }
@@ -284,9 +332,11 @@ static void updateValues()
   
   /* Values dependent on a visuData. */
   dataObj = toolPanelGet_visuData(TOOL_PANEL(panelBox));
+  DBG_fprintf(stderr, "Panel Box: update values for object %p.\n",
+	      (gpointer)dataObj);
   if (dataObj)
     {
-      if (visuDataGet_periodic(dataObj))
+      if (visuDataGet_boundaryConditions(dataObj) != BOX_FREE)
 	{
 	  transXYZ = visuDataGet_XYZtranslation(dataObj);
 	  visuDataConvert_XYZtoBoxCoordinates(dataObj, transBox, transXYZ);
@@ -306,6 +356,9 @@ static void updateValues()
 					(double)expand[i]);
 	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkAllowExpand), set);
 	}
+      gtk_combo_box_set_active(GTK_COMBO_BOX(comboUnit),
+			       visuDataGet_unit(dataObj));
+      gtk_widget_set_sensitive(comboUnit, TRUE);
     }
   else
     {
@@ -315,6 +368,7 @@ static void updateValues()
       for (i = 0; i < 3; i++)
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinExpandXYZ[i]), 0.);
       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkAllowExpand), FALSE);
+      gtk_widget_set_sensitive(comboUnit, FALSE);
     }
   disableCallbacks = FALSE;
   
@@ -329,7 +383,7 @@ static void updatePeriodic(VisuData *dataObj)
 {
   gboolean periodic;
 
-  periodic = (dataObj && visuDataGet_periodic(dataObj));
+  periodic = (dataObj && visuDataGet_boundaryConditions(dataObj) != BOX_FREE);
   gtk_widget_set_sensitive(vboxPeriodic, periodic);
   if (!periodic && dataObj)
     {
@@ -452,25 +506,21 @@ static void onStippleExpandChanged(StippleComboBox *combo _U_, gint stipple,
 static void onVisuDataChanged(GObject *obj _U_, VisuData* visuData, gpointer data _U_)
 {
   float cartCoord[3];
-  gboolean periodic;
   float transBox[3];
   int i;
+  VisuData *dataRef;
   
+  DBG_fprintf(stderr, "Panel Box: Catch 'dataLoaded' signal, update.\n");
+
   /* Update the sensitivity of the periodic stuff. */
   updatePeriodic(visuData);
 
   if (!visuData)
     return;
 
-  g_signal_connect(G_OBJECT(visuData), "ElementRenderedChanged",
-		   G_CALLBACK(onElementRenderChanged), (gpointer)0);
-
-  periodic = visuDataGet_periodic(visuData);
-  if (!periodic)
-    return;
-
   /* Apply the periodic stuffs. */
-  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkAllowTranslations)))
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkAllowTranslations)) &&
+      visuDataGet_boundaryConditions(visuData) != BOX_FREE)
     {
       DBG_fprintf(stderr, "Panel Box: Catch 'dataLoaded' signal,"
 		  " applying current translations.\n");
@@ -481,18 +531,38 @@ static void onVisuDataChanged(GObject *obj _U_, VisuData* visuData, gpointer dat
       visuDataSet_XYZtranslation(visuData, cartCoord);
       visuData_constrainedInTheBox(visuData);
     }
+
+  /* Make a geometry diff. */
+  dataRef = toolPanelGet_visuData(TOOL_PANEL(panelBox));
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkDiff)) &&
+      dataRef)
+    {
+      geometryDiff(dataRef, visuData);
+      geometryRebuild_list(visuData);
+    }
+
+  g_signal_connect(G_OBJECT(visuData), "ElementRenderedChanged",
+		   G_CALLBACK(onElementRenderChanged), (gpointer)0);
+  g_signal_connect(G_OBJECT(visuData), "NodePositionChanged",
+		   G_CALLBACK(onNodePositionChanged), (gpointer)0);
 }
 static void onVisuDataReady(GObject *obj _U_, VisuData* visuData, gpointer data _U_)
 {
-  gboolean periodic;
   float expand[3];
-  int i;
+  int i, val;
 
   if (!visuData)
     return;
 
-  periodic = visuDataGet_periodic(visuData);
-  if (!periodic)
+  /* Update the units if necessary. */
+  if (visuBasicGet_preferedUnit(visuData) == unit_undefined)
+    {
+      val = gtk_combo_box_get_active(GTK_COMBO_BOX(comboUnit));
+      if (val > 0 && visuDataGet_unit(visuData) != unit_undefined)
+	visuDataSet_unit(visuData, (VisuUnits)val);
+    }
+
+  if (visuDataGet_boundaryConditions(visuData) == BOX_FREE)
     return;
 
   /* Apply the periodic stuffs. */
@@ -651,3 +721,33 @@ static void onExpandChecked(GtkToggleButton* button, gpointer data _U_)
   gtk_widget_set_sensitive(spinTransXYZ[1], !set);
   gtk_widget_set_sensitive(spinTransXYZ[2], !set);
 }
+static void onUnitChanged(GtkComboBox *combo, gpointer data _U_)
+{
+  gint val;
+  VisuUnits unit;
+  VisuData *dataObj;
+
+  val = gtk_combo_box_get_active(combo);
+  g_return_if_fail(val >= 0);
+
+  unit = (VisuUnits)val;
+  
+  dataObj = toolPanelGet_visuData(TOOL_PANEL(panelBox));
+  g_return_if_fail(dataObj);
+
+  if (visuDataSet_unit(dataObj, unit))
+    {
+      visuData_createAllNodes(dataObj);
+      g_idle_add(visuObjectRedraw, (gpointer)0);
+    }
+}
+static void onDiffChanged(GtkToggleButton *toggle, gpointer data _U_)
+{
+  geometrySet_active(gtk_toggle_button_get_active(toggle));
+  g_idle_add(visuObjectRedraw, (gpointer)0);
+}
+static void onNodePositionChanged(VisuData *dataObj, gpointer data _U_)
+{
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkDiff)))
+    geometryRebuild_list(dataObj);
+}
