@@ -67,6 +67,7 @@ static GtkWidget *checkTransparency;
 static GtkWidget *checkStereo;
 static GtkWidget *spinPrecision;
 static GtkWidget *spinStereoAngle;
+static GtkWidget *hboxStereo;
 
 static int disableCallbacksOpenGL;
 static GtkListStore *light_list_store;
@@ -121,6 +122,7 @@ ToolPanel* panelOpenGL_init()
 
   /* Private parameters. */
   disableCallbacksOpenGL = 0;
+  hboxStereo = (GtkWidget*)0;
 
   return TOOL_PANEL(panelOpenGL);
 }
@@ -552,7 +554,7 @@ GtkWidget *createInteriorOpenGL()
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkStereo),
 			       openGLGet_stereo());
   gtk_box_pack_start(GTK_BOX(hbox), checkStereo, FALSE, FALSE, 2);
-  gtk_widget_set_sensitive(hbox, openGLGet_stereoCapability());
+  hboxStereo = hbox;
   /* Degrees. */
   label = gtk_label_new(_("deg."));
   gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 2);
@@ -746,6 +748,9 @@ static void onOpenGLEnter(ToolPanel *toolPanel _U_, gpointer data _U_)
   OpenGLExtension *ext;
 #endif
 
+  if (hboxStereo)
+    gtk_widget_set_sensitive(hboxStereo, openGLGet_stereoCapability());
+
   disableCallbacksOpenGL = 1;
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkImmediateDrawing),
 			       openGLGet_immediate());
