@@ -58,8 +58,7 @@
 #include <extraGtkFunctions/gtk_colorComboBoxWidget.h>
 #include <extraGtkFunctions/gtk_numericalEntryWidget.h>
 #include <extraGtkFunctions/gtk_orientationChooser.h>
-
-#include "visuConfig.h"
+#include <extraGtkFunctions/gtk_valueIOWidget.h>
 
 #define PANEL_PLANES_NO_PLANE_LIST _("<span font_desc=\"smaller\"><i>none</i></span>")
 
@@ -95,9 +94,10 @@ static void onPlaneListOpen(GtkButton *button, gpointer data);
 static void onPlaneListSave(GtkButton *button, gpointer data);
 static void onPlaneListSaveAs(GtkButton *button, gpointer data);
 
-gulong onAskForHide_signal_id;
-guint isPlayingDistanceId;
-gdouble directionDist;
+static gulong onAskForHide_signal_id;
+static guint isPlayingDistanceId;
+static gdouble directionDist;
+static gboolean disableCallbacks;
 
 /* Local callbacks. */
 void onPlaneUsed(GtkToggleButton *button, gpointer data);
@@ -105,8 +105,8 @@ void onPlaneAdd(GtkButton *button, gpointer data);
 void onPlaneRemove(GtkButton *button, gpointer data);
 static void onSpinNVectChange(GtkSpinButton *spin, gpointer data);
 static void onSpinDistanceChange(GtkSpinButton *spin, gpointer data);
-void onPlaneColorChange(ColorComboBox *combo, Color *selectedColor, gpointer data);
-void rebuildListPlanes(VisuData *dataObj);
+static void onPlaneColorChange(ColorComboBox *combo, Color *selectedColor, gpointer data);
+static void rebuildListPlanes(VisuData *dataObj);
 static void onDataReadyForRendering(GObject *obj, VisuData *dataObj, gpointer data);
 static void onNewFileLoaded(GObject *obj, VisuData *dataObj, gpointer data);
 void onTreeSelectionChanged(GtkTreeSelection *tree, gpointer data);
@@ -114,11 +114,10 @@ static void onNodePositionChanged(VisuData *dataObj, gpointer data);
 static void onNodePopulationIncreased(VisuData *dataObj,
 				      int *newNodes, gpointer data);
 void onSetCameraPosition(GtkButton *button, gpointer data);
-void onSpinBoundsChangeDistance(GObject *obj, gfloat val, gpointer data);
 static void onGtkPlanesDrawnToggled(GtkCellRendererToggle *cell_renderer,
 				    gchar *path, gpointer user_data);
-void onGtkPlanesHideToggled(GtkCellRendererToggle *cell_renderer,
-			    gchar *path, gpointer user_data);
+static void onGtkPlanesHideToggled(GtkCellRendererToggle *cell_renderer,
+				   gchar *path, gpointer user_data);
 void onGtkPlanesHidingModeToggled(GtkToggleButton *toggle, gpointer data);
 static void onAskForHideNodes(VisuData *visuData, gboolean *redraw, gpointer data);
 static void onPlayStopDist(GtkButton *button, gpointer data);
@@ -136,10 +135,10 @@ static gboolean playDistances(gpointer data);
 static void createAndInitPlanePanel(VisuData *dataObj);
 static void createInteriorPlanes(VisuData *dataObj);
 
-gboolean isPlanesInitialised;
+static gboolean isPlanesInitialised;
 
-OpenGLExtension *extensionPlanes;
-int identifierAllPlanes;
+static OpenGLExtension *extensionPlanes;
+static int identifierAllPlanes;
 
 /* String used to labelled planes, dist. means 'distance' and
    norm. means 'normal' (50 chars max). */
@@ -162,11 +161,12 @@ ToolPanel* panelPlanesInit()
   gtk_widget_set_sensitive(vBoxPlanes, FALSE);
 
   isPlanesInitialised = FALSE;
-  checkUsePlanes = (GtkWidget*)0;
-  spinbuttonDistance = (GtkWidget*)0;
-  orientationChooser = (GtkWidget*)0;
+  checkUsePlanes      = (GtkWidget*)0;
+  spinbuttonDistance  = (GtkWidget*)0;
+  orientationChooser  = (GtkWidget*)0;
   isPlayingDistanceId = 0;
   currentSaveListFile = (gchar*)0;
+  disableCallbacks    = FALSE;
   /* Create the listModel */
   listStoredPlanes = gtk_list_store_new (N_COLUMN_PLANE,
 					 G_TYPE_BOOLEAN,
@@ -199,49 +199,40 @@ GtkListStore* panelPlanesGet_listStore()
   return listStoredPlanes;
 }
 
-gboolean drawPlanes(GtkTreeModel *model, GtkTreePath *path _U_,
-		    GtkTreeIter *iter, gpointer data _U_)
+static void rebuildListPlanes(VisuData *dataObj _U_)
 {
+  Plane **planes;
+  int i;
+  GtkTreeIter iter;
+  gboolean valid;
   Plane *plane;
   gboolean used;
 
-  used = FALSE;
-  gtk_tree_model_get(model, iter, COLUMN_PLANE_DRAWN, &used, -1);
-  if (used)
+  DBG_fprintf(stderr, "Panel planes: rebuilding object list for planes.\n");
+
+  /* We create a list of planes. */
+  planes = g_malloc(sizeof(Plane*) *
+		    (gtk_tree_model_iter_n_children(GTK_TREE_MODEL(listStoredPlanes),
+						    (GtkTreeIter*)0) + 1));
+  i = 0;
+  for (valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(listStoredPlanes), &iter);
+       valid;
+       valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(listStoredPlanes), &iter))
     {
-      gtk_tree_model_get(model, iter, COLUMN_PLANE_POINTER, &plane, -1);
-      if (plane)
-	{
-	  /* Because the get action increase the counter... */
-	  g_object_unref(G_OBJECT(plane));
-	  planeDraw(plane);
-	}
-    }  
-  return FALSE;
-}
+      gtk_tree_model_get(GTK_TREE_MODEL(listStoredPlanes), &iter,
+			 COLUMN_PLANE_DRAWN, &used,
+			 COLUMN_PLANE_POINTER, &plane,
+			 -1);
+      /* Because the get action increase the counter... */
+      g_object_unref(G_OBJECT(plane));
+      if (used)
+	planes[i++] = plane;
+    }
+  planes[i] = (Plane*)0;
 
-void rebuildListPlanes(VisuData *dataObj)
-{
-  OpenGLView *view;
+  planesDraw_list(planes, identifierAllPlanes);
 
-  view = visuDataGet_openGLView(dataObj);
-  DBG_fprintf(stderr, "Panel planes : rebuilding object list for planes.\n");
-  glNewList(identifierAllPlanes, GL_COMPILE);
-
-  /* Set blend if not present. */
-  glEnable(GL_BLEND);
-  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-  glDisable(GL_LIGHTING);
-  glDisable(GL_DITHER);
-
-  glPushMatrix();
-  glTranslated(-view->box->dxxs2, -view->box->dyys2, -view->box->dzzs2);
-  gtk_tree_model_foreach(GTK_TREE_MODEL(listStoredPlanes), drawPlanes, (gpointer)0);
-  glPopMatrix();  
-
-  glEnable(GL_LIGHTING);
-  glEnable(GL_DITHER); /* WARNING: it is the default! */
-  glEndList();
+  g_free(planes);
 }
 
 static void createAndInitPlanePanel(VisuData *dataObj)
@@ -255,7 +246,8 @@ static void createAndInitPlanePanel(VisuData *dataObj)
 
 static void onPlanesEnter(ToolPanel *planes, gpointer data _U_)
 {
-  DBG_fprintf(stderr, "Panel Planes : caught the 'page-entered' signal.\n");
+  DBG_fprintf(stderr, "Panel Planes : caught the 'page-entered' signal %d.\n",
+	      isPlanesInitialised);
   if (!isPlanesInitialised)
     createAndInitPlanePanel(toolPanelGet_visuData(planes));
 }
@@ -285,15 +277,17 @@ static void createInteriorPlanes(VisuData *dataObj)
   GtkCellRenderer *renderer;
   GtkTreeViewColumn *column;
   GtkWidget *image;
-  GtkTooltips *tooltips;
   GSList *radiobuttonHideUnion_group;
   GtkWidget *radiobuttonHideInter, *radiobuttonHideUnion;
   GtkWidget *notebook;
   OpenGLView *view;
   int i;
   float span[2];
+#if GTK_MINOR_VERSION < 12
+  GtkTooltips *tooltips;
 
   tooltips = gtk_tooltips_new ();
+#endif
 
   gtk_widget_show(vBoxPlanes);
 
@@ -339,8 +333,8 @@ static void createInteriorPlanes(VisuData *dataObj)
   gtk_box_pack_start(GTK_BOX(hboxHidingMode), radiobuttonHideUnion, FALSE, FALSE, 0);
   gtk_radio_button_set_group(GTK_RADIO_BUTTON(radiobuttonHideUnion), (GSList*)0);
   radiobuttonHideUnion_group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(radiobuttonHideUnion));
-  gtk_tooltips_set_tip(tooltips, radiobuttonHideUnion,
-		       _("Hide all elements that are hidden by al least one plane."), NULL);
+  gtk_widget_set_tooltip_text(radiobuttonHideUnion,
+		       _("Hide all elements that are hidden by al least one plane."));
   hbox = gtk_hbox_new (FALSE, 2);
   gtk_widget_show(hbox);
   gtk_container_add(GTK_CONTAINER(radiobuttonHideUnion), hbox);
@@ -356,8 +350,8 @@ static void createInteriorPlanes(VisuData *dataObj)
   gtk_box_pack_start(GTK_BOX(hboxHidingMode), radiobuttonHideInter, FALSE, FALSE, 0);
   gtk_radio_button_set_group(GTK_RADIO_BUTTON(radiobuttonHideInter), radiobuttonHideUnion_group);
   radiobuttonHideUnion_group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(radiobuttonHideInter));
-  gtk_tooltips_set_tip(tooltips, radiobuttonHideInter,
-		       _("Hide elements only if they are hidden by all planes."), NULL);
+  gtk_widget_set_tooltip_text(radiobuttonHideInter,
+		       _("Hide elements only if they are hidden by all planes."));
   hbox = gtk_hbox_new (FALSE, 2);
   gtk_widget_show(hbox);
   gtk_container_add(GTK_CONTAINER(radiobuttonHideInter), hbox);
@@ -380,8 +374,8 @@ static void createInteriorPlanes(VisuData *dataObj)
   gtk_widget_show (vbox3);
   gtk_box_pack_start(GTK_BOX(hbox2), vbox3, FALSE, FALSE, 2);
   buttonRotate = gtk_button_new();
-  gtk_tooltips_set_tip(tooltips, buttonRotate, _("Set the camera to look in the direction"
-						 " of the normal of the selected plane."), NULL);
+  gtk_widget_set_tooltip_text(buttonRotate, _("Set the camera to look in the direction"
+						 " of the normal of the selected plane."));
   gtk_box_pack_start(GTK_BOX(vbox3), buttonRotate, TRUE, FALSE, 0);
   gtk_widget_show(buttonRotate);
   image = create_pixmap((GtkWidget*)0, "stock_rotate_20.png");
@@ -409,7 +403,7 @@ static void createInteriorPlanes(VisuData *dataObj)
 
   for (i = 0; i < 3; i++)
     {
-      spinbuttonNVect_adj[i] = gtk_adjustment_new (1, -999, 999, 1, 3, 3);
+      spinbuttonNVect_adj[i] = gtk_adjustment_new (1, -999, 999, 1, 3, 0);
       spinbuttonNVect[i] = gtk_spin_button_new(GTK_ADJUSTMENT
 					       (spinbuttonNVect_adj[i]), 1, 0);
       gtk_widget_show (spinbuttonNVect[i]);
@@ -432,16 +426,8 @@ static void createInteriorPlanes(VisuData *dataObj)
   gtk_box_pack_start (GTK_BOX (vbox3), hbox5, TRUE, TRUE, 0);
 
   labelDistance = gtk_label_new ("");
-  if (panelConfigGet_spinBoundsValue() == 1.)
-    gtk_label_set_text(GTK_LABEL(labelDistance),
-		       _("Distance from origin: "));
-  else
-    {
-      gtk_label_set_text(GTK_LABEL(labelDistance),
-			 _("Dist. from origin <span style=\"italic\" size=\"smaller\""
-			   " foreground=\"red\">\303\227 mult.</span>: "));
-      gtk_label_set_use_markup(GTK_LABEL(labelDistance), TRUE);
-    }
+  gtk_label_set_text(GTK_LABEL(labelDistance),
+		     _("Distance from origin: "));
   gtk_widget_show (labelDistance);
   gtk_box_pack_start (GTK_BOX (hbox5), labelDistance, TRUE, TRUE, 0);
   gtk_misc_set_alignment (GTK_MISC (labelDistance), 1, 0.5);
@@ -453,10 +439,10 @@ static void createInteriorPlanes(VisuData *dataObj)
       openGLViewGet_boxSpan(view, span);
       spinbuttonDistance_adj = gtk_adjustment_new((span[0] + span[1]) / 2.f,
 						  -span[1], span[1],
-						  0.25, 2, 2);
+						  0.25, 2, 0);
     }
   else
-    spinbuttonDistance_adj = gtk_adjustment_new(1, -1000, 1000, 0.25, 2, 2);
+    spinbuttonDistance_adj = gtk_adjustment_new(1, -1000, 1000, 0.25, 2, 0);
   spinbuttonDistance = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonDistance_adj), 1, 2);
   gtk_widget_show (spinbuttonDistance);
   gtk_box_pack_start (GTK_BOX (hbox5), spinbuttonDistance, FALSE, FALSE, 0);
@@ -559,9 +545,9 @@ static void createInteriorPlanes(VisuData *dataObj)
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
   buttonDistPlayStop = gtk_button_new();
-  gtk_tooltips_set_tip(tooltips, buttonDistPlayStop,
+  gtk_widget_set_tooltip_text(buttonDistPlayStop,
 		       _("Change the distance parameter of he selected file"
-			 " at the given rate."), NULL);
+			 " at the given rate."));
   gtk_widget_show(buttonDistPlayStop);
   gtk_box_pack_start(GTK_BOX(hbox), buttonDistPlayStop, FALSE, FALSE, 15);
   hbox = gtk_hbox_new(FALSE, 0);
@@ -661,9 +647,6 @@ static void createInteriorPlanes(VisuData *dataObj)
   g_signal_connect(G_OBJECT(buttonRotate), "clicked",
 		   G_CALLBACK(onSetCameraPosition), (gpointer)0);
 
-  g_signal_connect(G_OBJECT(gtkMainClassGet_currentPanel()), "spinBoundsChanged",
-		   G_CALLBACK(onSpinBoundsChangeDistance), (gpointer)0);
-
   g_signal_connect(G_OBJECT(radiobuttonHideUnion), "toggled",
 		   G_CALLBACK(onGtkPlanesHidingModeToggled), GINT_TO_POINTER(plane_hideUnion));
   g_signal_connect(G_OBJECT(radiobuttonHideInter), "toggled",
@@ -766,6 +749,9 @@ static void onAskForHideNodes(VisuData *visuData, gboolean *redraw,
 {
   Plane **listOfPlanes;
 
+  DBG_fprintf(stderr, "Panel Planes : caught the 'NodeAskForShowHide' signal for"
+	      " VisuData %p.\n", (gpointer)visuData);
+
   if (!checkUsePlanes || !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkUsePlanes)))
     return;
 
@@ -815,7 +801,7 @@ void onPlaneUsed(GtkToggleButton *button, gpointer data _U_)
 void onPlaneAdd(GtkButton *button _U_, gpointer userData _U_)
 {
   Plane* plane;
-  float vect[3];
+  float vect[3], vertices[8][3];
   int i;
   GtkTreeIter iter;
   Color *color;
@@ -831,7 +817,8 @@ void onPlaneAdd(GtkButton *button _U_, gpointer userData _U_)
   for (i = 0; i < 3; i++)
     vect[i] = (float)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbuttonNVect[i]));
   dist = (float)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbuttonDistance));
-  plane = planeNew(visuDataGet_openGLView(dataObj), vect, dist, color);
+  visuDataGet_boxVertices(dataObj, vertices, TRUE);
+  plane = planeNew(vertices, vect, dist, color);
   
   /* String used to labelled planes, dist. means 'distance' and
      norm. means 'normal' (50 chars max). */
@@ -848,7 +835,7 @@ void onPlaneAdd(GtkButton *button _U_, gpointer userData _U_)
   
   /* We count down the counter on plane since it has been added to the list. */
   g_object_unref(G_OBJECT(plane));
-
+  
   rebuildListPlanes((VisuData*)dataObj);
   gtk_tree_selection_select_iter(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeviewPlanes)), &iter);
   if (gtk_tree_model_iter_n_children(GTK_TREE_MODEL(listStoredPlanes), (GtkTreeIter*)0) > 1)
@@ -908,71 +895,67 @@ static void onSpinNVectChange(GtkSpinButton *spin, gpointer data)
   float vect[3];
   gboolean hide, side;
   int i;
-  int val;
   VisuData *dataObj;
 
   i = GPOINTER_TO_INT(data);
-  if (i < 0 || i > 2)
+  g_return_if_fail(i >= 0 && i < 3);
+
+  if (disableCallbacks)
     return;
 
   /* Get the VisuData to deal with. */
   dataObj = toolPanelGet_visuData(TOOL_PANEL(panelPlanes));
 
-  res = gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeviewPlanes)),
-					&tmpList, &iter);
-  if (res)
+  res = gtk_tree_selection_get_selected
+    (gtk_tree_view_get_selection(GTK_TREE_VIEW(treeviewPlanes)), &tmpList, &iter);
+
+  if (!res)
+    return;
+
+  plane = (Plane*)0;
+  hide = FALSE;
+  side = FALSE;
+  gtk_tree_model_get(GTK_TREE_MODEL(listStoredPlanes), &iter,
+		     COLUMN_PLANE_POINTER, &plane,
+		     COLUMN_PLANE_HIDE_IS_ON, &hide,
+		     COLUMN_PLANE_HIDDEN_SIDE, &side,
+		     -1);
+  g_return_if_fail(plane);
+  /* Because the get action increase the counter... */
+  g_object_unref(G_OBJECT(plane));
+
+  planeGet_nVectUser(plane, vect);
+  vect[i] = (float)gtk_spin_button_get_value(spin);
+  if (vect[0] == 0. && vect[1] == 0. && vect[2] == 0.)
     {
-      plane = (Plane*)0;
-      hide = FALSE;
-      side = FALSE;
-      gtk_tree_model_get(GTK_TREE_MODEL(listStoredPlanes), &iter,
-			 COLUMN_PLANE_POINTER, &plane,
-			 COLUMN_PLANE_HIDE_IS_ON, &hide,
-			 COLUMN_PLANE_HIDDEN_SIDE, &side,
-			 -1);
-      if (plane)
-	{
-	  /* Because the get action increase the counter... */
-	  g_object_unref(G_OBJECT(plane));
-
-	  planeGet_nVectUser(plane, vect);
-	  vect[i] = (float)gtk_spin_button_get_value(spin);
-	  val = planeSet_normalVector(plane, vect);
-	  if (val < 0)
-	    {
-	      g_warning("Can't assign the new normal vector: (%f; %f; %f).\n",
-			vect[0], vect[1], vect[2]);
-	      planeGet_nVectUser(plane, vect);
-	      for (i = 0; i < 3; i++)
-		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbuttonNVect[i]), vect[i]);
-	      return;
-	    }
-	  else if (val == 0)
-	    return;
-	  planeComputeInter(plane, visuDataGet_openGLView(dataObj));
-	  planeGet_distanceFromOrigin(plane, &dist);
-	  sprintf(str, LABEL_PLANE,
-		  (int)vect[0], (int)vect[1], (int)vect[2], dist);
-	  gtk_list_store_set(listStoredPlanes, &iter, COLUMN_PLANE_LABEL, str, -1);
-	  /* Redraw all planes. */
-	  rebuildListPlanes((VisuData*)dataObj);
-	  /* If extension is not used, we leave here. */
-	  if (!extensionPlanes->used)
-	    return;
-
-	  /* Need to modify the nodes visibility only if the current
-	     plane use the masking function (hiddenState != PLANE_SIDE_NONE). */
-	  if (planeGet_hiddenState(plane) != PLANE_SIDE_NONE)
-	    {
-	      /* Modify nodes visibility accordingly. */
-	      val = panelPlanesApply_hidingScheme(dataObj);
-	      if (val)
-		visuData_createAllNodes(dataObj);
-	    }
-	  if (extensionPlanes->used)
-	    g_idle_add(visuObjectRedraw, (gpointer)0);
-	}
+      g_warning("Can't assign a null vector.\n");
+      planeGet_nVectUser(plane, vect);
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbuttonNVect[0]), vect[0]);
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbuttonNVect[1]), vect[1]);
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbuttonNVect[2]), vect[2]);
+      return;
     }
+  res = planeSet_normalVector(plane, vect);
+  if (!res)
+    return;
+
+  planeGet_distanceFromOrigin(plane, &dist);
+  sprintf(str, LABEL_PLANE,
+	  (int)vect[0], (int)vect[1], (int)vect[2], dist);
+  gtk_list_store_set(listStoredPlanes, &iter, COLUMN_PLANE_LABEL, str, -1);
+  /* Redraw all planes. */
+  rebuildListPlanes((VisuData*)dataObj);
+
+  /* Need to modify the nodes visibility only if the current
+     plane use the masking function (hiddenState != PLANE_SIDE_NONE). */
+  if (extensionPlanes->used && planeGet_hiddenState(plane) != PLANE_SIDE_NONE)
+    {
+      /* Modify nodes visibility accordingly. */
+      res = panelPlanesApply_hidingScheme(dataObj);
+      if (res)
+	visuData_createAllNodes(dataObj);
+    }
+  g_idle_add(visuObjectRedraw, (gpointer)0);
 }
 static void onSpinDistanceChange(GtkSpinButton *spin, gpointer data _U_)
 {
@@ -981,86 +964,65 @@ static void onSpinDistanceChange(GtkSpinButton *spin, gpointer data _U_)
   GtkTreeIter iter;
   Plane *plane;
   char str[256];
-  float dist, mult;
+  float dist;
   float vect[3];
   gboolean hide, side;
   int val;
   VisuData *dataObj;
 
+  if (disableCallbacks)
+    return;
+
   /* Get the VisuData to deal with. */
   dataObj = toolPanelGet_visuData(TOOL_PANEL(panelPlanes));
 
   res = gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeviewPlanes)),
 					&tmpList, &iter);
-  if (res)
+  if (!res)
+    return;
+
+  plane = (Plane*)0;
+  hide = FALSE;
+  side = FALSE;
+  gtk_tree_model_get(GTK_TREE_MODEL(listStoredPlanes), &iter,
+		     COLUMN_PLANE_POINTER, &plane,
+		     COLUMN_PLANE_HIDE_IS_ON, &hide,
+		     COLUMN_PLANE_HIDDEN_SIDE, &side,
+		     -1);
+  g_return_if_fail(plane);
+  /* Because the get action increase the counter... */
+  g_object_unref(G_OBJECT(plane));
+
+  dist = (float)gtk_spin_button_get_value(spin);
+  val = planeSet_distanceFromOrigin(plane, dist);
+  if (val < 0)
     {
-      plane = (Plane*)0;
-      hide = FALSE;
-      side = FALSE;
-      gtk_tree_model_get(GTK_TREE_MODEL(listStoredPlanes), &iter,
-			 COLUMN_PLANE_POINTER, &plane,
-			 COLUMN_PLANE_HIDE_IS_ON, &hide,
-			 COLUMN_PLANE_HIDDEN_SIDE, &side,
-			 -1);
-      if (plane)
-	{
-	  /* Because the get action increase the counter... */
-	  g_object_unref(G_OBJECT(plane));
-
-	  mult = panelConfigGet_spinBoundsValue();
-	  dist = (float)gtk_spin_button_get_value(spin);
-	  val = planeSet_distanceFromOrigin(plane, dist * mult);
-	  if (val < 0)
-	    {
-	      g_warning("Can't assign the new distance from origin: %f.\n",
-			dist);
-	      gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbuttonDistance), 0.);
-	      return;
-	    }
-	  else if (val == 0)
-	    return;
-	  planeComputeInter(plane, visuDataGet_openGLView(dataObj));
-	  planeGet_nVectUser(plane, vect);
-	  sprintf(str, LABEL_PLANE,
-		  (int)vect[0], (int)vect[1], (int)vect[2], dist);
-	  gtk_list_store_set(listStoredPlanes, &iter, COLUMN_PLANE_LABEL, str, -1);
-	  /* Redraw all planes. */
-	  rebuildListPlanes((VisuData*)dataObj);
-	  /* If extension is not used, we leave here. */
-	  if (!extensionPlanes->used)
-	    return;
-
-	  /* Need to modify the nodes visibility only if the current
-	     plane use the masking function (hiddenState != PLANE_SIDE_NONE). */
-	  if (planeGet_hiddenState(plane) != PLANE_SIDE_NONE)
-	    {
-	      /* Modify nodes visibility accordingly. */
-	      val = panelPlanesApply_hidingScheme(dataObj);
-	      if (val)
-		visuData_createAllNodes(dataObj);
-	    }
-	  if (extensionPlanes->used)
-	    g_idle_add(visuObjectRedraw, (gpointer)0);
-	}
+      g_warning("Can't assign the new distance from origin: %f.\n",
+		dist);
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbuttonDistance), 0.);
+      return;
     }
-}
-void onSpinBoundsChangeDistance(GObject *obj _U_, gfloat mult, gpointer data _U_)
-{
-  DBG_fprintf(stderr, "Panel Plane : catch the 'spinBoundsChanged' signal,"
-	      " should adjust the distance from origin (multiplier : %f).\n", mult);
+  else if (val == 0)
+    return;
+  planeGet_nVectUser(plane, vect);
+  sprintf(str, LABEL_PLANE,
+	  (int)vect[0], (int)vect[1], (int)vect[2], dist);
+  gtk_list_store_set(listStoredPlanes, &iter, COLUMN_PLANE_LABEL, str, -1);
+  /* Redraw all planes. */
+  rebuildListPlanes((VisuData*)dataObj);
 
-  if (panelConfigGet_spinBoundsValue() == 1.)
-    gtk_label_set_text(GTK_LABEL(labelDistance),
-		       _("Distance from origin: "));
-  else
+  /* Need to modify the nodes visibility only if the current
+     plane use the masking function (hiddenState != PLANE_SIDE_NONE). */
+  if (extensionPlanes->used && planeGet_hiddenState(plane) != PLANE_SIDE_NONE)
     {
-      gtk_label_set_text(GTK_LABEL(labelDistance),
-			 _("Dist. from origin <span style=\"italic\" size=\"smaller\""
-			   " foreground=\"red\">\303\227 mult.</span>: "));
-      gtk_label_set_use_markup(GTK_LABEL(labelDistance), TRUE);
+      /* Modify nodes visibility accordingly. */
+      val = panelPlanesApply_hidingScheme(dataObj);
+      if (val)
+	visuData_createAllNodes(dataObj);
     }
+  g_idle_add(visuObjectRedraw, (gpointer)0);
 }
-void onPlaneColorChange(ColorComboBox *combo, Color *color, gpointer data _U_)
+static void onPlaneColorChange(ColorComboBox *combo, Color *color, gpointer data _U_)
 {
   gboolean res;
   GtkTreeModel *tmpList;
@@ -1070,6 +1032,9 @@ void onPlaneColorChange(ColorComboBox *combo, Color *color, gpointer data _U_)
   GdkPixbuf *pixbufColorAlphaBox;
   VisuData *dataObj;
 
+  if (disableCallbacks)
+    return;
+
   /* Get the VisuData to deal with. */
   dataObj = toolPanelGet_visuData(TOOL_PANEL(panelPlanes));
 
@@ -1117,8 +1082,7 @@ static void onDataReadyForRendering(GObject *obj _U_, VisuData *dataObj,
   Plane **list;
   int val, i;
   gboolean needShowHide;
-  OpenGLView *view;
-  float span[2];
+  float span[2], vertices[8][3];
 
   DBG_fprintf(stderr, "Panel Plane : caught 'dataReadyForRendering' signal\n"
 	      " | setting sensitivity.\n");
@@ -1130,13 +1094,10 @@ static void onDataReadyForRendering(GObject *obj _U_, VisuData *dataObj,
       return;
     }
 
-  view = visuDataGet_openGLView(dataObj);
-  g_return_if_fail(view);
-
   if (spinbuttonDistance)
     {
       DBG_fprintf(stderr, " | set the maximum of the distance spin.\n");
-      openGLViewGet_boxSpan(view, span);
+      openGLViewGet_boxSpan(visuDataGet_openGLView(dataObj), span);
       gtk_spin_button_set_range(GTK_SPIN_BUTTON(spinbuttonDistance),
 				-span[1], span[1]);
     }
@@ -1147,11 +1108,12 @@ static void onDataReadyForRendering(GObject *obj _U_, VisuData *dataObj,
   DBG_fprintf(stderr, " | recalculating intersections and masking set.\n");
 
   needShowHide = FALSE;
+  visuDataGet_boxVertices(dataObj, vertices, TRUE);
   list = panelPlanesGet_listOfAllPlanes(FALSE);
   for (i = 0; list[i]; i++)
     {
       /* Compute the intersection with the new box */
-      planeComputeInter(list[i], view);
+      planeSet_box(list[i], vertices);
       /* Test if one of the plane at least need to mask some nodes. */
       needShowHide = needShowHide ||
 	(planeGet_hiddenState(list[i]) != PLANE_SIDE_NONE);
@@ -1205,13 +1167,12 @@ static void onBoxSizeChanged(VisuData *data, gpointer user_data _U_)
   gboolean valid;
   GtkTreeIter iter;
   Plane *plane;
-  OpenGLView *view;
-  float span[2];
+  float span[2], vertices[8][3];
 
   DBG_fprintf(stderr, "Panel Plane : catch 'BoxSizeChanged' signal,"
 	      " recalculating plane sizes.\n");
 
-  view = visuDataGet_openGLView(data);
+  visuDataGet_boxVertices(data, vertices, TRUE);
   valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(listStoredPlanes), &iter);
   while (valid)
     {
@@ -1221,7 +1182,7 @@ static void onBoxSizeChanged(VisuData *data, gpointer user_data _U_)
 	{
 	  /* Because the get action increase the counter... */
 	  g_object_unref(G_OBJECT(plane));
-	  planeComputeInter(plane, view);
+	  planeSet_box(plane, vertices);
 	}
       valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(listStoredPlanes), &iter);
     }
@@ -1230,7 +1191,7 @@ static void onBoxSizeChanged(VisuData *data, gpointer user_data _U_)
   if (spinbuttonDistance)
     {
       DBG_fprintf(stderr, " | set the maximum of the distance spin.\n");
-      openGLViewGet_boxSpan(view, span);
+      openGLViewGet_boxSpan(visuDataGet_openGLView(data), span);
       gtk_spin_button_set_range(GTK_SPIN_BUTTON(spinbuttonDistance),
 				-span[1], span[1]);
     }
@@ -1243,11 +1204,10 @@ void onTreeSelectionChanged(GtkTreeSelection *tree, gpointer data _U_)
   GtkTreeModel *tmpList;
   GtkTreeIter iter;
   Plane *plane;
-  float val, nPlane[3], dist, mult;
-  int i;
+  float nPlane[3], dist;
   Color *colorPlane, *colorCombo;
 
-  DBG_fprintf(stderr, "Panel Plane : catch 'changed' signal from "
+  DBG_fprintf(stderr, "Panel Plane: catch 'changed' signal from "
 	      " the gtkTreeSelection.\n");
 
   res = gtk_tree_selection_get_selected(tree, &tmpList, &iter);
@@ -1260,22 +1220,22 @@ void onTreeSelectionChanged(GtkTreeSelection *tree, gpointer data _U_)
       /* Because the get action increase the counter... */
       g_object_unref(G_OBJECT(plane));
 
+      DBG_fprintf(stderr, "Panel Plane: get the values for plane %p.\n",
+		  (gpointer)plane);
       planeGet_nVectUser(plane, nPlane);
-      for (i = 0; i < 3; i++)
-	{
-	  val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbuttonNVect[i]));
-	  if (val != nPlane[i])
-	    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbuttonNVect[i]), nPlane[i]);
-	}
       planeGet_distanceFromOrigin(plane, &dist);
-      mult = panelConfigGet_spinBoundsValue();
-      val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbuttonDistance));
-      if (val != dist / mult)
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbuttonDistance), dist / mult);
       planeGet_color(plane, &colorPlane);
+
+      DBG_fprintf(stderr, "Panel Plane: update the widgets.\n");
+      disableCallbacks = TRUE;
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbuttonNVect[0]), nPlane[0]);
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbuttonNVect[1]), nPlane[1]);
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbuttonNVect[2]), nPlane[2]);
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbuttonDistance), dist);
       colorCombo = colorComboBoxGet_selectedColor(COLOR_COMBOX(widgetColorPlane));
-      if (colorPlane != colorCombo)
-	colorComboBoxSet_selectionByColor(COLOR_COMBOX(widgetColorPlane), colorPlane);
+      colorComboBoxSet_selectionByColor(COLOR_COMBOX(widgetColorPlane), colorPlane);
+      disableCallbacks = FALSE;
+      DBG_fprintf(stderr, " | done.\n");
 
       gtk_widget_set_sensitive(buttonRotate, TRUE);
       gtk_widget_set_sensitive(vboxDistChange, TRUE);
@@ -1341,6 +1301,7 @@ static void onGtkPlanesDrawnToggled(GtkCellRendererToggle *cell_renderer,
   g_return_if_fail(validIter);
   gtk_tree_model_get(GTK_TREE_MODEL(listStoredPlanes), &iter,
 		     COLUMN_PLANE_POINTER, &plane, -1);
+  g_object_unref(G_OBJECT(plane));
   panelSet_rendered(&iter, plane,
 		    !gtk_cell_renderer_toggle_get_active(cell_renderer));
   g_idle_add(visuObjectRedraw, (gpointer)0);
@@ -1357,6 +1318,7 @@ gboolean panelPlanesSet_planeRendered(Plane *plane, gboolean status)
     {
       gtk_tree_model_get(GTK_TREE_MODEL(listStoredPlanes), &iter,
 			 COLUMN_PLANE_POINTER, &storedPlane, -1);
+      g_object_unref(G_OBJECT(storedPlane));
       if (storedPlane == plane)
 	valid = FALSE;
       else
@@ -1376,6 +1338,7 @@ void onGtkPlanesHideToggled(GtkCellRendererToggle *cell_renderer _U_,
   Plane *plane;
   VisuData *dataObj;
 
+  DBG_fprintf(stderr, "Panel Planes: toggle plane hide mode.\n");
   validIter = gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(listStoredPlanes), &iter, path);
   g_return_if_fail(validIter);
 
@@ -1587,8 +1550,7 @@ gboolean panelPlanesLoad_file(VisuData *dataObj, gchar *filename, GError **error
 {
   Plane **list;
   gboolean res;
-  OpenGLView *view;
-  float vect[3];
+  float vect[3], vertices[8][3];
   float dist;
   Color *color;
   int hide, i;
@@ -1605,19 +1567,17 @@ gboolean panelPlanesLoad_file(VisuData *dataObj, gchar *filename, GError **error
   res = planesParse_XMLFile(filename, &list, error);
   if (!res)
     return FALSE;
-  DBG_fprintf(stderr, "Panel Planes : found planes in file '%s'.\n", filename);
-
-  view = visuDataGet_openGLView(dataObj);
-  g_return_val_if_fail(view, FALSE);
+  DBG_fprintf(stderr, "Panel Planes: found planes in file '%s'.\n", filename);
 
   /* Begin to compute intersections of all planes and then
      add then to the list. */
   gtk_list_store_clear(listStoredPlanes);
+  visuDataGet_boxVertices(dataObj, vertices, TRUE);
   for (i = 0; list[i]; i++)
     {
       DBG_fprintf(stderr, " | %d plane %p.\n", i, (gpointer)list[i]);
 
-      planeComputeInter(list[i], view);
+      planeSet_box(list[i], vertices);
 
       /* String used to labelled planes, dist. means 'distance' and
 	 norm. means 'normal' (50 chars max). */
@@ -1681,13 +1641,13 @@ static void onPlaneListOpen(GtkButton *button _U_, gpointer data _U_)
   gtk_window_set_modal(GTK_WINDOW(openDialog), TRUE);
   gtk_window_set_transient_for(GTK_WINDOW(openDialog), GTK_WINDOW(parent));
   gtk_window_set_position(GTK_WINDOW(openDialog), GTK_WIN_POS_CENTER_ON_PARENT);
-  directory = gtkMainClassGet_lastOpenDirectory();
+  directory = visuGtkGet_lastOpenDirectory();
   if (directory)
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(openDialog), directory);
   gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(openDialog), FALSE);
 
   filter = gtk_file_filter_new ();
-  gtk_file_filter_set_name(filter, _("Planes description (*.xml)"));
+  gtk_file_filter_set_name(filter, _("Plane description (*.xml)"));
   gtk_file_filter_add_pattern(filter, "*.xml");
   gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(openDialog), filter);
   filter = gtk_file_filter_new ();
@@ -1708,7 +1668,7 @@ static void onPlaneListOpen(GtkButton *button _U_, gpointer data _U_)
   if (error)
     {
       visuGtkRaise_warning(_("Loading a file"), error->message, (GtkWindow*)0);
-      g_clear_error(&error);
+      g_error_free(error);
       return;
     }
   /* If the extension is actually used, recompute the visibility of nodes. */
@@ -1752,48 +1712,14 @@ static void onPlaneListSave(GtkButton *button _U_, gpointer data _U_)
 }
 static void onPlaneListSaveAs(GtkButton *button _U_, gpointer data _U_)
 {
-  GtkWidget *saveDialog;
-  gchar *directory, *filename;
+  gchar *filename;
   gboolean res;
-  GtkWindow *parent;
-  GtkFileFilter *filter;
   
-  parent = toolPanelGet_containerWindow(TOOL_PANEL(panelPlanes));
-  saveDialog = gtk_file_chooser_dialog_new(_("Choose a file to save the list of planes"),
-                                           GTK_WINDOW(parent),
-					   GTK_FILE_CHOOSER_ACTION_SAVE,
-					   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					   GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-					   NULL);
-  gtk_window_set_modal(GTK_WINDOW(saveDialog), TRUE);
-  gtk_window_set_transient_for(GTK_WINDOW(saveDialog), GTK_WINDOW(parent));
-  gtk_window_set_position(GTK_WINDOW(saveDialog), GTK_WIN_POS_CENTER_ON_PARENT);
-  directory = gtkMainClassGet_lastOpenDirectory();
-  if (directory)
-    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(saveDialog), directory);
-  gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(saveDialog), FALSE);
-  gtk_dialog_set_default_response(GTK_DIALOG(saveDialog), GTK_RESPONSE_ACCEPT);
-
-  filter = gtk_file_filter_new ();
-  gtk_file_filter_set_name(filter, _("Planes description (*.xml)"));
-  gtk_file_filter_add_pattern(filter, "*.xml");
-  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(saveDialog), filter);
-  filter = gtk_file_filter_new ();
-  gtk_file_filter_set_name(filter, _("All files"));
-  gtk_file_filter_add_pattern (filter, "*");
-  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(saveDialog), filter);
-
-  /* A suggested filename. */
-  gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(saveDialog),
-				    _("planes.xml"));
-
-  if (gtk_dialog_run(GTK_DIALOG(saveDialog)) != GTK_RESPONSE_ACCEPT)
-    {
-      gtk_widget_destroy(saveDialog);
-      return;
-    }
-  filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(saveDialog));
-  gtk_widget_destroy(saveDialog);
+  filename =
+    valueIOGet_saveFilename(GTK_WINDOW(toolPanelGet_containerWindow
+				       (TOOL_PANEL(panelPlanes))));
+  if (!filename)
+    return;
 
   res = callbackSave(filename);
   if (res)
