@@ -41,6 +41,9 @@
 	knowledge of the CeCILL license and that you accept its terms. You can
 	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
 */
+#include <glib.h>
+#include <glib/gstdio.h>
+
 #include "renderingAtomic.h"
 #include "renderingAtomic_ascii.h"
 #include "renderingAtomic_d3.h"
@@ -55,10 +58,6 @@
 #include <coreTools/toolConfigFile.h>
 #include <coreTools/toolElements.h>
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
 #include <GL/gl.h>
 #include <GL/glu.h> 
 
@@ -69,8 +68,6 @@
    describe the spheres. */
 /* int identifierSpheres; */
 
-static RenderingMethod* pointerOnRenderingAtomicMethod;
-
 /* The icosahedron drawing. */
 #define X .525731112119133606 
 #define Z .850650808352039932
@@ -96,14 +93,12 @@ static GLuint tindices[20][3] = {
 static gboolean readAtomicRadiusShape(gchar **lines, int nbLines, int position,
 			      VisuData *dataObj, GError **error);
 /* These functions write all the element list to export there associated resources. */
-static gboolean exportAtomicRadiusShape(GString *data, int *nbLinesWritten,
-					VisuData* dataObj);
+static void exportAtomicRadiusShape(GString *data, VisuData* dataObj);
 #define FLAG_PARAMETER_SHAPE "atomic_sphere_method"
 #define DESC_PARAMETER_SHAPE "The sphere drawing method, [GluSphere Icosahedron]"
 static gboolean readAtomicShape(gchar **lines, int nbLines, int position,
 				VisuData *dataObj, GError **error);
-static gboolean exportAtomicShape(GString *data, int *nbLinesWritten,
-				  VisuData* dataObj);
+static void exportAtomicShape(GString *data, VisuData* dataObj);
 enum
   {
     sphere_glu,
@@ -113,7 +108,7 @@ enum
 static guint sphereMethod = sphere_icosahedron;
 const char* sphereName[sphere_nb + 1] = {"GluSphere", "Icosahedron", (const char*)0};
 
-const char* shapeName[nbAtomicShapes + 1] = {"Sphere", "Cube", "Elipsoid", "Point", (const char*)0};
+const char* shapeName[nbAtomicShapes + 1] = {"Sphere", "Cube", "Elipsoid", "Point", "Torus", (const char*)0};
 const char* shapeNameI18n[nbAtomicShapes + 1];
 struct atomicResources_struct
 {
@@ -135,15 +130,16 @@ struct atomicResources_struct
 };
 
 
+static VisuRendering *atom;
 /* To build a list of rendering loading methods. */
 /* This list contains all the method than can load an atomic file. */
 static GList *allAtomicLoadingMethods;
+static VisuRendering* atomic = (VisuRendering*)0;
 
 static struct atomicResources_struct *getRadiusAndShape(VisuElement *ele);
 
-RenderingMethod* initAtomic()
+void renderingAtomicInit()
 {
-  RenderingMethod *atom;
   char *name = _("Atom visualisation");
   char *descr = _("It draws spheres at specified positions to represent atoms."
 		  " The radius of the sphere can vary.");
@@ -166,7 +162,8 @@ RenderingMethod* initAtomic()
   meth = atomicXyzInit();
   if (meth)
     allAtomicLoadingMethods = g_list_prepend(allAtomicLoadingMethods, meth);
-  allAtomicLoadingMethods = g_list_sort(allAtomicLoadingMethods, renderingMethodCompare_priority);
+  allAtomicLoadingMethods = g_list_sort(allAtomicLoadingMethods,
+					visuRenderingFormatCompare_priority);
   tmpLst = allAtomicLoadingMethods;
   while (tmpLst)
     {
@@ -174,15 +171,15 @@ RenderingMethod* initAtomic()
       tmpLst = g_list_next(tmpLst);
     }
 
-  atom = renderingMethod_new("Atom visualisation", name, descr,
-			     1, renderingAtomicLoad);
-  renderingMethodSet_fileType(atom, formatList, 0, _("Position files"));
+  atom = visuRendering_new(VISU_RENDERING_ATOMIC, name, descr,
+			   1, renderingAtomicLoad,
+			   renderingAtomic_createShape,
+			   renderingAtomic_positionShape,
+			   renderingAtomicGet_radius);
+  visuRenderingSet_fileType(atom, formatList, 0, _("Position files"));
   iconPath = g_build_filename(V_SIM_PIXMAPS_DIR, "stock-atomic.png", NULL);
-  renderingMethodSet_icon(atom, iconPath);
+  visuRenderingSet_icon(atom, iconPath);
   g_free(iconPath);
-  setOpenGLMethods(atom, renderingAtomic_createShape,
-		   renderingAtomic_positionShape,
-		   renderingAtomicGet_radius);
 
   /* Dealing with config files. */
   resourceEntry = visuConfigFileAdd_entry(VISU_CONFIGFILE_RESOURCE,
@@ -204,15 +201,14 @@ RenderingMethod* initAtomic()
 /*   identifierSpheres = openGLObjectList_new(NMAX_TP); */
 
   /* Set the general pointer to retrieve easily this method. */
-  pointerOnRenderingAtomicMethod = atom;
+  atomic = atom;
 
   shapeNameI18n[0] = _("Sphere");
   shapeNameI18n[1] = _("Cube");
   shapeNameI18n[2] = _("Elipsoid");
   shapeNameI18n[3] = _("Point");
-  shapeNameI18n[4] = (const char*)0;
-
-  return atom;
+  shapeNameI18n[4] = _("Torus");
+  shapeNameI18n[5] = (const char*)0;
 }
 
 gboolean renderingAtomicLoad(VisuData *data, FileFormat *format _U_,
@@ -222,6 +218,10 @@ gboolean renderingAtomicLoad(VisuData *data, FileFormat *format _U_,
   gboolean loadOk;
   GList *tmpLst;
   FileFormat *fmt;
+  RenderingFormatLoad *load;
+#if GLIB_MINOR_VERSION > 5
+  struct stat buf;
+#endif
 
   g_return_val_if_fail(error && *error == (GError*)0, FALSE);
 
@@ -236,14 +236,22 @@ gboolean renderingAtomicLoad(VisuData *data, FileFormat *format _U_,
       return FALSE;
     }
   
-  if (g_file_test(file, G_FILE_TEST_IS_DIR))
+  if (!g_file_test(file, G_FILE_TEST_IS_REGULAR))
     {
       *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FILE,
 			   _("The specified file is"
-			     " a directory indeed. Can't load"
-			     " a directory!\n"));
+			     " not a regular file."));
       return FALSE;
     }
+#if GLIB_MINOR_VERSION > 5
+  if (!g_stat(file, &buf) && buf.st_size == 0)
+    {
+      *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FILE,
+			   _("The specified file is"
+			     " an empty file."));
+      return FALSE;
+    }
+#endif
   
   loadOk = FALSE;
   tmpLst = allAtomicLoadingMethods;
@@ -254,21 +262,30 @@ gboolean renderingAtomicLoad(VisuData *data, FileFormat *format _U_,
       if (*error)
 	g_error_free(*error);
       *error = (GError*)0;
-      if (!fmt || ((RenderingFormatLoad*)tmpLst->data)->fmt == fmt)
+      load = (RenderingFormatLoad*)tmpLst->data;
+      if (!fmt || load->fmt == fmt)
 	{
-	  DBG_fprintf(stderr,"Rendering Atomic : testing '%s' with format : %s.\n",
-		      file, ((RenderingFormatLoad*)tmpLst->data)->name);
-	  loadOk = ((RenderingFormatLoad*)tmpLst->data)->load
-	    (data, file, ((RenderingFormatLoad*)tmpLst->data)->fmt, nSet, error);
+	  DBG_fprintf(stderr,"Rendering Atomic: testing '%s' with format: %s.\n",
+		      file, load->name);
+	  loadOk = load->load(data, file, load->fmt, nSet, error);
+	  DBG_fprintf(stderr,"Rendering Atomic: with format '%s' returns %d.\n",
+		      load->name, loadOk);
 	}
-      tmpLst = g_list_next(tmpLst);
+      if (*error && (*error)->domain == G_FILE_ERROR)
+	return FALSE;
+      
+      if (fmt && load->fmt == fmt)
+	tmpLst = (GList*)0;
+      else
+	tmpLst = g_list_next(tmpLst);
     }
 
   if (!loadOk)
     {
-      if (!*error)
-	*error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FILE,
-			     _("Impossible to load this file.\n"));
+      if (*error)
+	g_error_free(*error);
+      *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FILE,
+			   _("Impossible to load this file, unrecognised format.\n"));
       return FALSE;
     }
   else if (*error)
@@ -329,6 +346,8 @@ gboolean renderingAtomicSet_radius(VisuElement* ele, float value)
   else
     str->radius = value;
 
+  visuRenderingEmit_elementSize(atom, value);
+
   /* Ask for reDraw if needed. */
   return TRUE;
 }
@@ -428,7 +447,7 @@ gboolean renderingAtomicSet_elipsoidParameters(VisuElement *ele, float ratio,
       refresh = TRUE;
     }
 
-  return refresh && (str->shape == shapeElipsoid);
+  return refresh && (str->shape == shapeElipsoid || str->shape == shapeTorus);
 }
 gboolean renderingAtomicSet_elipsoidRatio(VisuElement *ele, float ratio)
 {
@@ -442,7 +461,7 @@ gboolean renderingAtomicSet_elipsoidRatio(VisuElement *ele, float ratio)
   if (ratio != str->ratio)
     {
       str->ratio = ratio;
-      return (str->shape == shapeElipsoid);
+      return (str->shape == shapeElipsoid || str->shape == shapeTorus);
     }
   return FALSE;
 }
@@ -458,7 +477,7 @@ gboolean renderingAtomicSet_elipsoidPhi(VisuElement *ele, float phi)
   if (phi != str->phi)
     {
       str->phi = phi;
-      return (str->shape == shapeElipsoid);
+      return (str->shape == shapeElipsoid || str->shape == shapeTorus);
     }
   return FALSE;
 }
@@ -474,7 +493,7 @@ gboolean renderingAtomicSet_elipsoidTheta(VisuElement *ele, float theta)
   if (theta != str->theta)
     {
       str->theta = theta;
-      return (str->shape == shapeElipsoid);
+      return (str->shape == shapeElipsoid || str->shape == shapeTorus);
     }
   return FALSE;
 }
@@ -575,15 +594,13 @@ static gboolean readAtomicRadiusShape(gchar **lines, int nbLines, int position,
 }
 
 /* These functions write all the element list to export there associated resources. */
-static gboolean exportAtomicRadiusShape(GString *data, int *nbLinesWritten,
-					VisuData *dataObj)
+static void exportAtomicRadiusShape(GString *data, VisuData *dataObj)
 {
   GList *pos, *eleList;
   unsigned int i;
   struct atomicResources_struct *str;
 
   g_string_append_printf(data, "# %s\n", DESC_RESOURCE_RADIUS_SHAPE);
-  *nbLinesWritten = 1;
   /* We create a list of elements, or get the whole list. */
   eleList = (GList*)0;
   if (dataObj)
@@ -604,16 +621,12 @@ static gboolean exportAtomicRadiusShape(GString *data, int *nbLinesWritten,
 	  g_string_append_printf(data, "    %s %10.3f %s\n",
 				 ((VisuElement*)pos->data)->name,
 				 str->radius, shapeName[str->shape]);
-	  *nbLinesWritten += 2;
 	}
       pos = g_list_next(pos);
     }
   g_string_append_printf(data, "\n");
-  *nbLinesWritten += 1;
   if (eleList)
     g_list_free(eleList);
-
-  return TRUE;
 }
 
 static gboolean readAtomicShape(gchar **lines, int nbLines, int position,
@@ -644,15 +657,11 @@ static gboolean readAtomicShape(gchar **lines, int nbLines, int position,
   g_strfreev(vals);
   return TRUE;
 }
-static gboolean exportAtomicShape(GString *data, int *nbLinesWritten,
-				  VisuData* dataObj _U_)
+static void exportAtomicShape(GString *data, VisuData* dataObj _U_)
 {
   g_string_append_printf(data, "# %s\n", DESC_PARAMETER_SHAPE);
   g_string_append_printf(data, "%s: %s\n\n", FLAG_PARAMETER_SHAPE,
 			 sphereName[sphereMethod]);
-  *nbLinesWritten += 3;
-
-  return TRUE;
 }
 
 
@@ -664,16 +673,19 @@ void renderingAtomic_positionShape(VisuData *visuData, VisuNode *node, VisuEleme
 {
   float rgba[4];
   float xyz[3];
+  float scale;
 
   visuDataGet_nodePosition(visuData, node, xyz);
+  scale = visuDataGet_nodeScalingFactor(visuData, node);
 
   glPushMatrix();
   glTranslated(xyz[0], xyz[1], xyz[2]);
   if (visuData->setColor)
     {
       visuData->setColor(visuData, rgba, ele, node);
-      setOpenGlMaterial(ele->material, rgba);
+      openGLSet_color(ele->material, rgba);
     }
+  glScalef(scale, scale, scale);
   glCallList(ele->openGLIdentifier);
   glPopMatrix();
 }
@@ -732,7 +744,7 @@ int renderingAtomic_createShape(VisuData *visuData, VisuElement* ele)
   view = visuDataGet_openGLView(visuData);
   nlat = OpenGLViewGet_numberOfFacettes(view, str->radius);
 
-  DBG_fprintf(stderr, "Rendering Atomic : creating '%s' for %s (%d - OpenGL id"
+  DBG_fprintf(stderr, "Rendering Atomic: creating '%s' for %s (%d - OpenGL id"
 	      " %d - fac %d)\n", shapeName[str->shape],
 	      ele->name, ele->typeNumber, str->openGLIdentifier,
 	      nlat);
@@ -821,27 +833,36 @@ int renderingAtomic_createShape(VisuData *visuData, VisuElement* ele)
 
       glEnd();
       break;
+    case shapeTorus:
+      
+      glPushMatrix();
+      glRotatef(str->phi, 0., 0., 1.);
+      glRotatef(str->theta, 0., 1., 0.);
+      openGLObjectListDraw_torus(obj, 0, str->radius, str->ratio, nlat, nlat, FALSE);
+      glPopMatrix();
+      
+      break;
+	
     }
   glEndList();
 
   gluDeleteQuadric(obj);
-  
+   
 /*   ele->openGLIdentifier = identifierSpheres + ele->typeNumber; */
+  DBG_fprintf(stderr, " | shape created %d\n", str->openGLIdentifier);
   return str->openGLIdentifier;
 }
 
 void renderingAtomicAdd_loadMethod(RenderingFormatLoad* meth)
 {
   g_return_if_fail(meth && meth->load);
+  g_return_if_fail(atomic);
 
-  DBG_fprintf(stderr, "Rendering Atomic : adding a new loading method '%s'.\n", meth->name);
+  DBG_fprintf(stderr, "Rendering Atomic: adding a new loading method '%s'.\n",
+	      meth->name);
   allAtomicLoadingMethods = g_list_prepend(allAtomicLoadingMethods, meth);
-  allAtomicLoadingMethods = g_list_sort(allAtomicLoadingMethods, renderingMethodCompare_priority);
+  allAtomicLoadingMethods = g_list_sort(allAtomicLoadingMethods,
+					visuRenderingFormatCompare_priority);
   if (meth->fmt)
-    renderingMethodAdd_fileFormat(pointerOnRenderingAtomicMethod, meth->fmt, 0);
-}
-
-RenderingMethod* renderingAtomicGet_static()
-{
-  return pointerOnRenderingAtomicMethod;
+    visuRenderingAdd_fileFormat(atomic, meth->fmt, 0);
 }
